@@ -2,7 +2,7 @@
 // This code is governed by the BSD license found in the LICENSE file.
 
 /*---
-info: >
+info: |
     Number instances have no special properties beyond those
     inherited from the Number prototype object
 es5id: 15.7.5_A1_T04
@@ -10,13 +10,13 @@ description: Checking property valueOf
 ---*/
 
 //CHECK#1
-if((new Number()).hasOwnProperty("valueOf") !== false){
-  $ERROR('#1: Number instance must have no special property "valueOf"');
+if ((new Number()).hasOwnProperty("valueOf") !== false) {
+  throw new Test262Error('#1: Number instance must have no special property "valueOf"');
 }
 
 //CHECK#2
-if((new Number()).valueOf !== Number.prototype.valueOf){
-  $ERROR('#2: Number instance property "valueOf" must be inherited from Number prototype object');
+if ((new Number()).valueOf !== Number.prototype.valueOf) {
+  throw new Test262Error('#2: Number instance property "valueOf" must be inherited from Number prototype object');
 }
 
 reportCompare(0, 0);
