@@ -2,7 +2,7 @@
 // This code is governed by the BSD license found in the LICENSE file.
 
 /*---
-info: >
+info: |
     Number instances have no special properties beyond those
     inherited from the Number prototype object
 es5id: 15.7.5_A1_T05
@@ -10,13 +10,13 @@ description: Checking property toFixed
 ---*/
 
 //CHECK#1
-if((new Number()).hasOwnProperty("toFixed") !== false){
-  $ERROR('#1: Number instance must have no special property "toFixed"');
+if ((new Number()).hasOwnProperty("toFixed") !== false) {
+  throw new Test262Error('#1: Number instance must have no special property "toFixed"');
 }
 
 //CHECK#2
-if((new Number()).toFixed !== Number.prototype.toFixed){
-  $ERROR('#2: Number instance property "toFixed" must be inherited from Number prototype object');
+if ((new Number()).toFixed !== Number.prototype.toFixed) {
+  throw new Test262Error('#2: Number instance property "toFixed" must be inherited from Number prototype object');
 }
 
 reportCompare(0, 0);
