@@ -2,23 +2,23 @@
 // This code is governed by the BSD license found in the LICENSE file.
 
 /*---
-info: >
+info: |
     Since calling Object as a function is identical to calling a function,
     list of arguments bracketing is allowed
 es5id: 15.2.2.1_A6_T2
 description: Creating an object with "new Object(null,2,3)"
 ---*/
 
-var obj = new Object(null,2,3);
+var obj = new Object(null, 2, 3);
 
 //CHECK#1
 if (obj.constructor !== Object) {
-  $ERROR('#1: Since Object as a function calling is the same as function calling list of arguments can appears in braces;');
+  throw new Test262Error('#1: Since Object as a function calling is the same as function calling list of arguments can appears in braces;');
 }
 
 //CHECK#2
 if (typeof obj !== "object") {
-  $ERROR('#2: Since Object as a function calling is the same as function calling list of arguments can appears in braces;');
+  throw new Test262Error('#2: Since Object as a function calling is the same as function calling list of arguments can appears in braces;');
 }
 
 reportCompare(0, 0);
