@@ -2,7 +2,7 @@
 // This code is governed by the BSD license found in the LICENSE file.
 
 /*---
-info: >
+info: |
     The Object.prototype.propertyIsEnumerable.length property has the
     attribute ReadOnly
 es5id: 15.2.4.7_A10
@@ -14,16 +14,18 @@ includes: [propertyHelper.js]
 
 //CHECK#1
 if (!(Object.prototype.propertyIsEnumerable.hasOwnProperty('length'))) {
-  $ERROR('#1: the Object.prototype.propertyIsEnumerable has length property');
+  throw new Test262Error('#1: the Object.prototype.propertyIsEnumerable has length property');
 }
 
 var obj = Object.prototype.propertyIsEnumerable.length;
 
-verifyNotWritable(Object.prototype.propertyIsEnumerable, "length", null, function(){return "shifted";});
+verifyNotWritable(Object.prototype.propertyIsEnumerable, "length", null, function() {
+  return "shifted";
+});
 
 //CHECK#2
 if (Object.prototype.propertyIsEnumerable.length !== obj) {
-  $ERROR('#2: the Object.prototype.propertyIsEnumerable length property has the attributes ReadOnly');
+  throw new Test262Error('#2: the Object.prototype.propertyIsEnumerable length property has the attributes ReadOnly');
 }
 
 reportCompare(0, 0);
