@@ -2,7 +2,7 @@
 // This code is governed by the BSD license found in the LICENSE file.
 
 /*---
-info: >
+info: |
     When the propertyIsEnumerable method is called with argument V, the following steps are taken:
     i) Let O be this object
     ii) Call ToString(V)
@@ -17,28 +17,30 @@ description: >
 
 //CHECK#1
 if (typeof Object.prototype.propertyIsEnumerable !== "function") {
-  $ERROR('#1: propertyIsEnumerable method is defined');
+  throw new Test262Error('#1: propertyIsEnumerable method is defined');
 }
 
-var obj = {the_property:true};
+var obj = {
+  the_property: true
+};
 
 //CHECK#2
 if (typeof obj.propertyIsEnumerable !== "function") {
-  $ERROR('#2: propertyIsEnumerable method is accessed');
+  throw new Test262Error('#2: propertyIsEnumerable method is accessed');
 }
 
 //CHECK#3
 if (!(obj.propertyIsEnumerable("the_property"))) {
-  $ERROR('#3: propertyIsEnumerable method works properly');
+  throw new Test262Error('#3: propertyIsEnumerable method works properly');
 }
 
 //CHECK#4
-var accum="";
-for(var prop in obj) {
-  accum+=prop;
+var accum = "";
+for (var prop in obj) {
+  accum += prop;
 }
-if (accum.indexOf("the_property")!==0) {
-  $ERROR('#4: enumerating works properly');
+if (accum.indexOf("the_property") !== 0) {
+  throw new Test262Error('#4: enumerating works properly');
 }
 //
 
