@@ -1,8 +1,9 @@
+// |reftest| async
 // Copyright 2014 Cubane Canada, Inc.  All rights reserved.
 // See LICENSE for details.
 
 /*---
-info: >
+info: |
     [...]
     5. Let rejectResult be Call(promiseCapability.[[Reject]], undefined, «r»).
     [...]
@@ -19,13 +20,13 @@ flags: [async]
 var p = Promise.reject(3);
 
 if (!(p instanceof Promise)) {
-    $ERROR("Expected Promise.reject to return a promise.");
+  throw new Test262Error("Expected Promise.reject to return a promise.");
 }
 
-p.then(function () {
-    $ERROR("Promise should not be fulfilled.");
-}, function (arg) {
-    if (arg !== 3) {
-        $ERROR("Expected promise to be rejected with supplied arg, got " + arg);
-    }
+p.then(function() {
+  throw new Test262Error("Promise should not be fulfilled.");
+}, function(arg) {
+  if (arg !== 3) {
+    throw new Test262Error("Expected promise to be rejected with supplied arg, got " + arg);
+  }
 }).then($DONE, $DONE);
