@@ -2,13 +2,9 @@
 // This code is governed by the BSD license found in the LICENSE file.
 
 /*---
-info: >
-    If string.charAt(k) in [0xE000 - 0xFFFF], return 3 octets (xxxxyyyy
-    yyzzzzzz -> 1110xxxx 10yyyyyy 10zzzzzz)
-es5id: 15.1.3.3_A2.5_T1
-es6id: 18.2.6.4
+info: If string.charAt(k) in [0xDC00 - 0xDFFF], throw URIError
 esid: sec-encodeuri-uri
-description: Complex tests, use RFC 3629
+description: Complex tests
 includes: [decimalToHexString.js]
 ---*/
 
@@ -16,14 +12,15 @@ var errorCount = 0;
 var count = 0;
 var indexP;
 var indexO = 0;
-for (var index = 0xE000; index <= 0xFFFF; index++) {
-  count++;
-  var hex1 = decimalToPercentHexString(0x0080 + (index & 0x003F));
-  var hex2 = decimalToPercentHexString(0x0080 + (index & 0x0FC0) / 0x0040);
-  var hex3 = decimalToPercentHexString(0x00E0 + (index & 0xF000) / 0x1000);
-  var str = String.fromCharCode(index);
-  if (encodeURI(str).toUpperCase() === hex3 + hex2 + hex1) continue;
 
+for (var index = 0xDC00; index <= 0xDFFF; index++) {
+  count++;
+  var hex = decimalToHexString(index);
+  try {
+    encodeURI(String.fromCharCode(index));
+  } catch (e) {
+    if ((e instanceof URIError) === true) continue;
+  }
   if (indexO === 0) {
     indexO = index;
   } else {
@@ -31,11 +28,11 @@ for (var index = 0xE000; index <= 0xFFFF; index++) {
       if ((indexP - indexO) !== 0) {
         var hexP = decimalToHexString(indexP);
         var hexO = decimalToHexString(indexO);
-        $ERROR('#' + hexO + '-' + hexP + ' ');
+        throw new Test262Error('#' + hexO + '-' + hexP + ' ');
       }
       else {
         var hexP = decimalToHexString(indexP);
-        $ERROR('#' + hexP + ' ');
+        throw new Test262Error('#' + hexP + ' ');
       }
       indexO = index;
     }
@@ -48,12 +45,12 @@ if (errorCount > 0) {
   if ((indexP - indexO) !== 0) {
     var hexP = decimalToHexString(indexP);
     var hexO = decimalToHexString(indexO);
-    $ERROR('#' + hexO + '-' + hexP + ' ');
+    throw new Test262Error('#' + hexO + '-' + hexP + ' ');
   } else {
     var hexP = decimalToHexString(indexP);
-    $ERROR('#' + hexP + ' ');
+    throw new Test262Error('#' + hexP + ' ');
   }
-  $ERROR('Total error: ' + errorCount + ' bad Unicode character in ' + count + ' ');
+  throw new Test262Error('Total error: ' + errorCount + ' bad Unicode character in ' + count + ' ');
 }
 
 reportCompare(0, 0);
