@@ -2,7 +2,7 @@
 // This code is governed by the BSD license found in the LICENSE file.
 
 /*---
-info: >
+info: |
     If multi line comments csn not nest, they can contain any Unicode
     character
 es5id: 7.4_A6
@@ -18,18 +18,18 @@ for (var indexI = 0; indexI <= 65535; indexI++) {
     eval("/*var " + String.fromCharCode(indexI) + "xx = 1*/");
     var hex = decimalToHexString(indexI);
     if (xx !== 0) {
-      $ERROR('#' + hex + ' ');
+      throw new Test262Error('#' + hex + ' ');
       errorCount++;
     }
   } catch (e){
-    $ERROR('#' + hex + ' ');
+    throw new Test262Error('#' + hex + ' ');
     errorCount++;
   }
   count++;
 }
 
 if (errorCount > 0) {
-  $ERROR('Total error: ' + errorCount + ' bad Unicode character in ' + count);
+  throw new Test262Error('Total error: ' + errorCount + ' bad Unicode character in ' + count);
 }
 
 function decimalToHexString(n) {
