@@ -2,7 +2,7 @@
 // This code is governed by the BSD license found in the LICENSE file.
 
 /*---
-info: >
+info: |
     "throw Expression" returns (throw, GetValue(Result(1)), empty), where 1
     evaluates Expression
 es5id: 12.13_A2_T5
@@ -14,7 +14,7 @@ try{
   throw 13;
 }
 catch(e){
-  if (e!==13) $ERROR('#1: Exception ===13. Actual:  Exception ==='+ e );
+  if (e!==13) throw new Test262Error('#1: Exception ===13. Actual:  Exception ==='+ e );
 }
 
 // CHECK#2
@@ -23,7 +23,7 @@ try{
   throw b;
 }
 catch(e){
-  if (e!==13) $ERROR('#2: Exception ===13. Actual:  Exception ==='+ e );
+  if (e!==13) throw new Test262Error('#2: Exception ===13. Actual:  Exception ==='+ e );
 }
 
 // CHECK#3
@@ -31,7 +31,7 @@ try{
   throw 2.13;
 }
 catch(e){
-  if (e!==2.13) $ERROR('#3: Exception ===2.13. Actual:  Exception ==='+ e );
+  if (e!==2.13) throw new Test262Error('#3: Exception ===2.13. Actual:  Exception ==='+ e );
 }
 
 // CHECK#4
@@ -47,7 +47,7 @@ try{
   throw +Infinity;
 }
 catch(e){
-  if (e!==+Infinity) $ERROR('#5: Exception ===+Infinity. Actual:  Exception ==='+ e );
+  if (e!==+Infinity) throw new Test262Error('#5: Exception ===+Infinity. Actual:  Exception ==='+ e );
 }
 
 // CHECK#6
@@ -55,7 +55,7 @@ try{
   throw -Infinity;
 }
 catch(e){
-  if (e!==-Infinity) $ERROR('#6: Exception ===-Infinity. Actual:  Exception ==='+ e );
+  if (e!==-Infinity) throw new Test262Error('#6: Exception ===-Infinity. Actual:  Exception ==='+ e );
 }
 
 // CHECK#7
@@ -63,7 +63,7 @@ try{
   throw +0;
 }
 catch(e){
-  if (e!==+0) $ERROR('#7: Exception ===+0. Actual:  Exception ==='+ e );
+  if (e!==+0) throw new Test262Error('#7: Exception ===+0. Actual:  Exception ==='+ e );
 }
 
 // CHECK#8
@@ -71,7 +71,7 @@ try{
   throw -0;
 }
 catch(e){
-  if (e!==-0) $ERROR('#8: Exception ===-0. Actual:  Exception ==='+ e );
+  if (e!==-0) throw new Test262Error('#8: Exception ===-0. Actual:  Exception ==='+ e );
 }
 
 reportCompare(0, 0);
