@@ -1,5 +1,5 @@
 /* -*- Mode: C++; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*- */
-/* vim: set sw=2 ts=2 et tw=79: */
+/* vim: set sw=2 ts=2 et tw=80: */
 /* This Source Code Form is subject to the terms of the Mozilla Public
  * License, v. 2.0. If a copy of the MPL was not distributed with this
  * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
@@ -8,7 +8,7 @@
 
 #include "mozilla/AutoRestore.h"
 #include "nsCRT.h"
-#include "nsContentUtils.h" // for kLoadAsData
+#include "nsContentUtils.h"  // for kLoadAsData
 #include "nsHtml5AtomTable.h"
 #include "nsHtml5DependentUTF16Buffer.h"
 #include "nsHtml5Tokenizer.h"
@@ -32,31 +32,30 @@ NS_IMPL_CYCLE_COLLECTION_TRAVERSE_END
 
 NS_IMPL_CYCLE_COLLECTION_UNLINK_BEGIN(nsHtml5Parser)
   NS_IMPL_CYCLE_COLLECTION_UNLINK(mExecutor)
+  NS_IMPL_CYCLE_COLLECTION_UNLINK_WEAK_REFERENCE
   tmp->DropStreamParser();
 NS_IMPL_CYCLE_COLLECTION_UNLINK_END
 
 nsHtml5Parser::nsHtml5Parser()
-  : mLastWasCR(false)
-  , mDocWriteSpeculativeLastWasCR(false)
-  , mBlocked(0)
-  , mDocWriteSpeculatorActive(false)
-  , mInsertionPointPushLevel(0)
-  , mDocumentClosed(false)
-  , mInDocumentWrite(false)
-  , mInsertionPointPermanentlyUndefined(false)
-  , mFirstBuffer(new nsHtml5OwningUTF16Buffer((void*)nullptr))
-  , mLastBuffer(mFirstBuffer)
-  , mExecutor(new nsHtml5TreeOpExecutor())
-  , mTreeBuilder(new nsHtml5TreeBuilder(mExecutor, nullptr))
-  , mTokenizer(new nsHtml5Tokenizer(mTreeBuilder, false))
-  , mRootContextLineNumber(1)
-  , mReturnToStreamParserPermitted(false)
-{
+    : mLastWasCR(false),
+      mDocWriteSpeculativeLastWasCR(false),
+      mBlocked(0),
+      mDocWriteSpeculatorActive(false),
+      mScriptNestingLevel(0),
+      mDocumentClosed(false),
+      mInDocumentWrite(false),
+      mInsertionPointPermanentlyUndefined(false),
+      mFirstBuffer(new nsHtml5OwningUTF16Buffer((void*)nullptr)),
+      mLastBuffer(mFirstBuffer),
+      mExecutor(new nsHtml5TreeOpExecutor()),
+      mTreeBuilder(new nsHtml5TreeBuilder(mExecutor, nullptr)),
+      mTokenizer(new nsHtml5Tokenizer(mTreeBuilder.get(), false)),
+      mRootContextLineNumber(1),
+      mReturnToStreamParserPermitted(false) {
   mTokenizer->setInterner(&mAtomTable);
 }
 
-nsHtml5Parser::~nsHtml5Parser()
-{
+nsHtml5Parser::~nsHtml5Parser() {
   mTokenizer->end();
   if (mDocWriteSpeculativeTokenizer) {
     mDocWriteSpeculativeTokenizer->end();
@@ -64,53 +63,48 @@ nsHtml5Parser::~nsHtml5Parser()
 }
 
 NS_IMETHODIMP_(void)
-nsHtml5Parser::SetContentSink(nsIContentSink* aSink)
-{
+nsHtml5Parser::SetContentSink(nsIContentSink* aSink) {
   NS_ASSERTION(aSink == static_cast<nsIContentSink*>(mExecutor),
                "Attempt to set a foreign sink.");
 }
 
 NS_IMETHODIMP_(nsIContentSink*)
-nsHtml5Parser::GetContentSink()
-{
+nsHtml5Parser::GetContentSink() {
   return static_cast<nsIContentSink*>(mExecutor);
 }
 
 NS_IMETHODIMP_(void)
-nsHtml5Parser::GetCommand(nsCString& aCommand)
-{
+nsHtml5Parser::GetCommand(nsCString& aCommand) {
   aCommand.AssignLiteral("view");
 }
 
 NS_IMETHODIMP_(void)
-nsHtml5Parser::SetCommand(const char* aCommand)
-{
+nsHtml5Parser::SetCommand(const char* aCommand) {
   NS_ASSERTION(!strcmp(aCommand, "view") || !strcmp(aCommand, "view-source") ||
-                 !strcmp(aCommand, "external-resource") ||
-                 !strcmp(aCommand, "import") || !strcmp(aCommand, kLoadAsData),
+                   !strcmp(aCommand, "external-resource") ||
+                   !strcmp(aCommand, "import") ||
+                   !strcmp(aCommand, kLoadAsData),
                "Unsupported parser command");
 }
 
 NS_IMETHODIMP_(void)
-nsHtml5Parser::SetCommand(eParserCommands aParserCommand)
-{
+nsHtml5Parser::SetCommand(eParserCommands aParserCommand) {
   NS_ASSERTION(aParserCommand == eViewNormal,
                "Parser command was not eViewNormal.");
 }
 
-void
-nsHtml5Parser::SetDocumentCharset(NotNull<const Encoding*> aEncoding,
-                                  int32_t aCharsetSource)
-{
+void nsHtml5Parser::SetDocumentCharset(NotNull<const Encoding*> aEncoding,
+                                       int32_t aCharsetSource,
+                                       bool aForceAutoDetection) {
   MOZ_ASSERT(!mExecutor->HasStarted(), "Document charset set too late.");
   MOZ_ASSERT(GetStreamParser(), "Setting charset on a script-only parser.");
-  GetStreamParser()->SetDocumentCharset(aEncoding, aCharsetSource);
+  GetStreamParser()->SetDocumentCharset(aEncoding, aCharsetSource,
+                                        aForceAutoDetection);
   mExecutor->SetDocumentCharsetAndSource(aEncoding, aCharsetSource);
 }
 
 NS_IMETHODIMP
-nsHtml5Parser::GetChannel(nsIChannel** aChannel)
-{
+nsHtml5Parser::GetChannel(nsIChannel** aChannel) {
   if (GetStreamParser()) {
     return GetStreamParser()->GetChannel(aChannel);
   } else {
@@ -119,34 +113,26 @@ nsHtml5Parser::GetChannel(nsIChannel** aChannel)
 }
 
 NS_IMETHODIMP
-nsHtml5Parser::GetDTD(nsIDTD** aDTD)
-{
+nsHtml5Parser::GetDTD(nsIDTD** aDTD) {
   *aDTD = nullptr;
   return NS_OK;
 }
 
-nsIStreamListener*
-nsHtml5Parser::GetStreamListener()
-{
+nsIStreamListener* nsHtml5Parser::GetStreamListener() {
   return mStreamListener;
 }
 
 NS_IMETHODIMP
-nsHtml5Parser::ContinueInterruptedParsing()
-{
-  NS_NOTREACHED("Don't call. For interface compat only.");
+nsHtml5Parser::ContinueInterruptedParsing() {
+  MOZ_ASSERT_UNREACHABLE("Don't call. For interface compat only.");
   return NS_ERROR_NOT_IMPLEMENTED;
 }
 
 NS_IMETHODIMP_(void)
-nsHtml5Parser::BlockParser()
-{
-  mBlocked++;
-}
+nsHtml5Parser::BlockParser() { mBlocked++; }
 
 NS_IMETHODIMP_(void)
-nsHtml5Parser::UnblockParser()
-{
+nsHtml5Parser::UnblockParser() {
   MOZ_DIAGNOSTIC_ASSERT(mBlocked > 0);
   if (MOZ_LIKELY(mBlocked > 0)) {
     mBlocked--;
@@ -157,31 +143,20 @@ nsHtml5Parser::UnblockParser()
 }
 
 NS_IMETHODIMP_(void)
-nsHtml5Parser::ContinueInterruptedParsingAsync()
-{
+nsHtml5Parser::ContinueInterruptedParsingAsync() {
   if (mExecutor) {
     mExecutor->ContinueInterruptedParsingAsync();
   }
 }
 
 NS_IMETHODIMP_(bool)
-nsHtml5Parser::IsParserEnabled()
-{
-  return !mBlocked;
-}
+nsHtml5Parser::IsParserEnabled() { return !mBlocked; }
 
 NS_IMETHODIMP_(bool)
-nsHtml5Parser::IsComplete()
-{
-  return mExecutor->IsComplete();
-}
+nsHtml5Parser::IsComplete() { return mExecutor->IsComplete(); }
 
 NS_IMETHODIMP
-nsHtml5Parser::Parse(nsIURI* aURL,
-                     nsIRequestObserver* aObserver,
-                     void* aKey,      // legacy; ignored
-                     nsDTDMode aMode) // legacy; ignored
-{
+nsHtml5Parser::Parse(nsIURI* aURL, void* /* legacy; ignored */) {
   /*
    * Do NOT cause WillBuildModel to be called synchronously from here!
    * The document won't be ready for it until OnStartRequest!
@@ -191,20 +166,14 @@ nsHtml5Parser::Parse(nsIURI* aURL,
   MOZ_ASSERT(GetStreamParser(),
              "Can't call this Parse() variant on script-created parser");
 
-  GetStreamParser()->SetObserver(aObserver);
-  GetStreamParser()->SetViewSourceTitle(aURL); // In case we're viewing source
+  GetStreamParser()->SetViewSourceTitle(aURL);  // In case we're viewing source
   mExecutor->SetStreamParser(GetStreamParser());
   mExecutor->SetParser(this);
   return NS_OK;
 }
 
-nsresult
-nsHtml5Parser::Parse(const nsAString& aSourceBuffer,
-                     void* aKey,
-                     const nsACString& aContentType,
-                     bool aLastCall,
-                     nsDTDMode aMode) // ignored
-{
+nsresult nsHtml5Parser::Parse(const nsAString& aSourceBuffer, void* aKey,
+                              bool aLastCall) {
   nsresult rv;
   if (NS_FAILED(rv = mExecutor->IsBroken())) {
     return rv;
@@ -220,38 +189,10 @@ nsHtml5Parser::Parse(const nsAString& aSourceBuffer,
   // Gripping the other objects just in case, since the other old grip
   // required grips to these, too.
   RefPtr<nsHtml5StreamParser> streamKungFuDeathGrip(GetStreamParser());
-  mozilla::Unused << streamKungFuDeathGrip; // Not used within function
+  mozilla::Unused << streamKungFuDeathGrip;  // Not used within function
   RefPtr<nsHtml5TreeOpExecutor> executor(mExecutor);
 
-  if (!executor->HasStarted()) {
-    NS_ASSERTION(!GetStreamParser(),
-                 "Had stream parser but document.write started life cycle.");
-    // This is the first document.write() on a document.open()ed document
-    executor->SetParser(this);
-    mTreeBuilder->setScriptingEnabled(executor->IsScriptEnabled());
-
-    bool isSrcdoc = false;
-    nsCOMPtr<nsIChannel> channel;
-    rv = GetChannel(getter_AddRefs(channel));
-    if (NS_SUCCEEDED(rv)) {
-      isSrcdoc = NS_IsSrcdocChannel(channel);
-    }
-    mTreeBuilder->setIsSrcdocDocument(isSrcdoc);
-
-    mTokenizer->start();
-    executor->Start();
-    if (!aContentType.EqualsLiteral("text/html")) {
-      mTreeBuilder->StartPlainText();
-      mTokenizer->StartPlainText();
-    }
-    /*
-     * If you move the following line, be very careful not to cause
-     * WillBuildModel to be called before the document has had its
-     * script global object set.
-     */
-    rv = executor->WillBuildModel(eDTDMode_unknown);
-    NS_ENSURE_SUCCESS(rv, rv);
-  }
+  MOZ_RELEASE_ASSERT(executor->HasStarted());
 
   // Return early if the parser has processed EOF
   if (executor->IsComplete()) {
@@ -277,8 +218,8 @@ nsHtml5Parser::Parse(const nsAString& aSourceBuffer,
   // document.writeln call--not document.close().
 
   MOZ_RELEASE_ASSERT(
-    IsInsertionPointDefined(),
-    "Doc.write reached parser with undefined insertion point.");
+      IsInsertionPointDefined(),
+      "Doc.write reached parser with undefined insertion point.");
 
   MOZ_RELEASE_ASSERT(!(GetStreamParser() && !aKey),
                      "Got a null key in a non-script-created parser");
@@ -339,7 +280,7 @@ nsHtml5Parser::Parse(const nsAString& aSourceBuffer,
         if (prevSearchBuf->next == mLastBuffer) {
           // key was not found
           nsHtml5OwningUTF16Buffer* keyHolder =
-            new nsHtml5OwningUTF16Buffer(aKey);
+              new nsHtml5OwningUTF16Buffer(aKey);
           keyHolder->next = mFirstBuffer;
           mFirstBuffer = keyHolder;
           prevSearchBuf = nullptr;
@@ -351,7 +292,7 @@ nsHtml5Parser::Parse(const nsAString& aSourceBuffer,
         }
         prevSearchBuf = prevSearchBuf->next;
       }
-    } // else mFirstBuffer is the keyholder
+    }  // else mFirstBuffer is the keyholder
 
     // prevSearchBuf is the previous buffer before the keyholder or null if
     // there isn't one.
@@ -397,8 +338,8 @@ nsHtml5Parser::Parse(const nsAString& aSourceBuffer,
       }
 
       if (mTreeBuilder->HasScript()) {
-        mTreeBuilder->Flush();               // Move ops to the executor
-        rv = executor->FlushDocumentWrite(); // run the ops
+        mTreeBuilder->Flush();                // Move ops to the executor
+        rv = executor->FlushDocumentWrite();  // run the ops
         NS_ENSURE_SUCCESS(rv, rv);
         // Flushing tree ops can cause all sorts of things.
         // Return early if the parser got terminated.
@@ -431,8 +372,8 @@ nsHtml5Parser::Parse(const nsAString& aSourceBuffer,
       // prevSearchBuf (which may be null)
       if (mFirstBuffer->key == aKey) {
         NS_ASSERTION(
-          !prevSearchBuf,
-          "Non-null prevSearchBuf when mFirstBuffer is the key holder?");
+            !prevSearchBuf,
+            "Non-null prevSearchBuf when mFirstBuffer is the key holder?");
         heapBuffer->next = mFirstBuffer;
         mFirstBuffer = heapBuffer;
       } else {
@@ -453,12 +394,12 @@ nsHtml5Parser::Parse(const nsAString& aSourceBuffer,
     }
   }
 
-  if (!mBlocked) { // buffer was tokenized to completion
+  if (!mBlocked) {  // buffer was tokenized to completion
     NS_ASSERTION(!stackBuffer.hasMore(),
                  "Buffer wasn't tokenized to completion?");
     // Scripting semantics require a forced tree builder flush here
-    mTreeBuilder->Flush();               // Move ops to the executor
-    rv = executor->FlushDocumentWrite(); // run the ops
+    mTreeBuilder->Flush();                // Move ops to the executor
+    rv = executor->FlushDocumentWrite();  // run the ops
     NS_ENSURE_SUCCESS(rv, rv);
   } else if (stackBuffer.hasMore()) {
     // The buffer wasn't tokenized to completion. Tokenize the untokenized
@@ -469,16 +410,16 @@ nsHtml5Parser::Parse(const nsAString& aSourceBuffer,
       if (!mDocWriteSpeculativeTreeBuilder) {
         // Lazily initialize if uninitialized
         mDocWriteSpeculativeTreeBuilder =
-          new nsHtml5TreeBuilder(nullptr, executor->GetStage());
+            MakeUnique<nsHtml5TreeBuilder>(nullptr, executor->GetStage());
         mDocWriteSpeculativeTreeBuilder->setScriptingEnabled(
-          mTreeBuilder->isScriptingEnabled());
-        mDocWriteSpeculativeTokenizer =
-          new nsHtml5Tokenizer(mDocWriteSpeculativeTreeBuilder, false);
+            mTreeBuilder->isScriptingEnabled());
+        mDocWriteSpeculativeTokenizer = MakeUnique<nsHtml5Tokenizer>(
+            mDocWriteSpeculativeTreeBuilder.get(), false);
         mDocWriteSpeculativeTokenizer->setInterner(&mAtomTable);
         mDocWriteSpeculativeTokenizer->start();
       }
       mDocWriteSpeculativeTokenizer->resetToDataState();
-      mDocWriteSpeculativeTreeBuilder->loadState(mTreeBuilder, &mAtomTable);
+      mDocWriteSpeculativeTreeBuilder->loadState(mTreeBuilder.get());
       mDocWriteSpeculativeLastWasCR = false;
     }
 
@@ -495,11 +436,11 @@ nsHtml5Parser::Parse(const nsAString& aSourceBuffer,
       stackBuffer.adjust(mDocWriteSpeculativeLastWasCR);
       if (stackBuffer.hasMore()) {
         if (!mDocWriteSpeculativeTokenizer->EnsureBufferSpace(
-              stackBuffer.getLength())) {
+                stackBuffer.getLength())) {
           return executor->MarkAsBroken(NS_ERROR_OUT_OF_MEMORY);
         }
         mDocWriteSpeculativeLastWasCR =
-          mDocWriteSpeculativeTokenizer->tokenizeBuffer(&stackBuffer);
+            mDocWriteSpeculativeTokenizer->tokenizeBuffer(&stackBuffer);
         nsresult rv;
         if (NS_FAILED((rv = mDocWriteSpeculativeTreeBuilder->IsBroken()))) {
           return executor->MarkAsBroken(rv);
@@ -516,8 +457,9 @@ nsHtml5Parser::Parse(const nsAString& aSourceBuffer,
 }
 
 NS_IMETHODIMP
-nsHtml5Parser::Terminate()
-{
+nsHtml5Parser::Terminate() {
+  // Prevent a second call to DidBuildModel via document.close()
+  mDocumentClosed = true;
   // We should only call DidBuildModel once, so don't do anything if this is
   // the second time that Terminate has been called.
   if (mExecutor->IsComplete()) {
@@ -536,53 +478,38 @@ nsHtml5Parser::Terminate()
 
 NS_IMETHODIMP
 nsHtml5Parser::ParseFragment(const nsAString& aSourceBuffer,
-                             nsTArray<nsString>& aTagStack)
-{
+                             nsTArray<nsString>& aTagStack) {
   return NS_ERROR_NOT_IMPLEMENTED;
 }
 
 NS_IMETHODIMP
-nsHtml5Parser::BuildModel()
-{
-  NS_NOTREACHED("Don't call this!");
+nsHtml5Parser::BuildModel() {
+  MOZ_ASSERT_UNREACHABLE("Don't call this!");
   return NS_ERROR_NOT_IMPLEMENTED;
 }
 
 NS_IMETHODIMP
-nsHtml5Parser::CancelParsingEvents()
-{
-  NS_NOTREACHED("Don't call this!");
+nsHtml5Parser::CancelParsingEvents() {
+  MOZ_ASSERT_UNREACHABLE("Don't call this!");
   return NS_ERROR_NOT_IMPLEMENTED;
 }
 
-void
-nsHtml5Parser::Reset()
-{
-  NS_NOTREACHED("Don't call this!");
-}
+void nsHtml5Parser::Reset() { MOZ_ASSERT_UNREACHABLE("Don't call this!"); }
 
-bool
-nsHtml5Parser::IsInsertionPointDefined()
-{
+bool nsHtml5Parser::IsInsertionPointDefined() {
   return !mExecutor->IsFlushing() && !mInsertionPointPermanentlyUndefined &&
-         (!GetStreamParser() || mInsertionPointPushLevel);
+         (!GetStreamParser() || mScriptNestingLevel != 0);
 }
 
-void
-nsHtml5Parser::PushDefinedInsertionPoint()
-{
-  ++mInsertionPointPushLevel;
+void nsHtml5Parser::IncrementScriptNestingLevel() { ++mScriptNestingLevel; }
+
+void nsHtml5Parser::DecrementScriptNestingLevel() { --mScriptNestingLevel; }
+
+bool nsHtml5Parser::HasNonzeroScriptNestingLevel() const {
+  return mScriptNestingLevel != 0;
 }
 
-void
-nsHtml5Parser::PopDefinedInsertionPoint()
-{
-  --mInsertionPointPushLevel;
-}
-
-void
-nsHtml5Parser::MarkAsNotScriptCreated(const char* aCommand)
-{
+void nsHtml5Parser::MarkAsNotScriptCreated(const char* aCommand) {
   MOZ_ASSERT(!mStreamListener, "Must not call this twice.");
   eParserMode mode = NORMAL;
   if (!nsCRT::strcmp(aCommand, "view-source")) {
@@ -599,27 +526,21 @@ nsHtml5Parser::MarkAsNotScriptCreated(const char* aCommand)
 #ifdef DEBUG
   else {
     NS_ASSERTION(!nsCRT::strcmp(aCommand, "view") ||
-                   !nsCRT::strcmp(aCommand, "external-resource") ||
-                   !nsCRT::strcmp(aCommand, "import"),
+                     !nsCRT::strcmp(aCommand, "external-resource") ||
+                     !nsCRT::strcmp(aCommand, "import"),
                  "Unsupported parser command!");
   }
 #endif
   mStreamListener =
-    new nsHtml5StreamListener(new nsHtml5StreamParser(mExecutor, this, mode));
+      new nsHtml5StreamListener(new nsHtml5StreamParser(mExecutor, this, mode));
 }
 
-bool
-nsHtml5Parser::IsScriptCreated()
-{
-  return !GetStreamParser();
-}
+bool nsHtml5Parser::IsScriptCreated() { return !GetStreamParser(); }
 
 /* End nsIParser  */
 
 // not from interface
-nsresult
-nsHtml5Parser::ParseUntilBlocked()
-{
+nsresult nsHtml5Parser::ParseUntilBlocked() {
   nsresult rv = mExecutor->IsBroken();
   NS_ENSURE_SUCCESS(rv, rv);
   if (mBlocked || mInsertionPointPermanentlyUndefined ||
@@ -643,8 +564,8 @@ nsHtml5Parser::ParseUntilBlocked()
           PermanentlyUndefineInsertionPoint();
           nsresult rv;
           MOZ_RELEASE_ASSERT(
-            !GetStreamParser(),
-            "This should only happen with script-created parser.");
+              !GetStreamParser(),
+              "This should only happen with script-created parser.");
           if (NS_SUCCEEDED((rv = mExecutor->IsBroken()))) {
             mTokenizer->eof();
             if (NS_FAILED((rv = mTreeBuilder->IsBroken()))) {
@@ -669,7 +590,7 @@ nsHtml5Parser::ParseUntilBlocked()
             mTreeBuilder->Flush();
             mReturnToStreamParserPermitted = false;
             GetStreamParser()->ContinueAfterScripts(
-              mTokenizer, mTreeBuilder, mLastWasCR);
+                mTokenizer.get(), mTreeBuilder.get(), mLastWasCR);
           }
         } else {
           // Script-created parser
@@ -679,7 +600,7 @@ nsHtml5Parser::ParseUntilBlocked()
           NS_ASSERTION(mExecutor->IsInFlushLoop(),
                        "How did we come here without being in the flush loop?");
         }
-        return NS_OK; // no more data for now but expecting more
+        return NS_OK;  // no more data for now but expecting more
       }
       mFirstBuffer = mFirstBuffer->next;
       continue;
@@ -720,19 +641,33 @@ nsHtml5Parser::ParseUntilBlocked()
   }
 }
 
-nsresult
-nsHtml5Parser::Initialize(nsIDocument* aDoc,
-                          nsIURI* aURI,
-                          nsISupports* aContainer,
-                          nsIChannel* aChannel)
-{
+nsresult nsHtml5Parser::StartExecutor() {
+  MOZ_ASSERT(!GetStreamParser(),
+             "Had stream parser but document.write started life cycle.");
+  // This is part of the setup document.open() does.
+  RefPtr<nsHtml5TreeOpExecutor> executor(mExecutor);
+  executor->SetParser(this);
+  mTreeBuilder->setScriptingEnabled(executor->IsScriptEnabled());
+
+  mTreeBuilder->setIsSrcdocDocument(false);
+
+  mTokenizer->start();
+  executor->Start();
+
+  /*
+   * We know we're in document.open(), so our document must already
+   * have a script global andthe WillBuildModel call is safe.
+   */
+  return executor->WillBuildModel(eDTDMode_unknown);
+}
+
+nsresult nsHtml5Parser::Initialize(mozilla::dom::Document* aDoc, nsIURI* aURI,
+                                   nsISupports* aContainer,
+                                   nsIChannel* aChannel) {
   return mExecutor->Init(aDoc, aURI, aContainer, aChannel);
 }
 
-void
-nsHtml5Parser::StartTokenizer(bool aScriptingEnabled)
-{
-
+void nsHtml5Parser::StartTokenizer(bool aScriptingEnabled) {
   bool isSrcdoc = false;
   nsCOMPtr<nsIChannel> channel;
   nsresult rv = GetChannel(getter_AddRefs(channel));
@@ -746,22 +681,18 @@ nsHtml5Parser::StartTokenizer(bool aScriptingEnabled)
   mTokenizer->start();
 }
 
-void
-nsHtml5Parser::InitializeDocWriteParserState(nsAHtml5TreeBuilderState* aState,
-                                             int32_t aLine)
-{
+void nsHtml5Parser::InitializeDocWriteParserState(
+    nsAHtml5TreeBuilderState* aState, int32_t aLine) {
   mTokenizer->resetToDataState();
   mTokenizer->setLineNumber(aLine);
-  mTreeBuilder->loadState(aState, &mAtomTable);
+  mTreeBuilder->loadState(aState);
   mLastWasCR = false;
   mReturnToStreamParserPermitted = true;
 }
 
-void
-nsHtml5Parser::ContinueAfterFailedCharsetSwitch()
-{
+void nsHtml5Parser::ContinueAfterFailedCharsetSwitch() {
   MOZ_ASSERT(
-    GetStreamParser(),
-    "Tried to continue after failed charset switch without a stream parser");
+      GetStreamParser(),
+      "Tried to continue after failed charset switch without a stream parser");
   GetStreamParser()->ContinueAfterFailedCharsetSwitch();
 }
