@@ -12,9 +12,7 @@
 // be available on a 32-bit system.
 #define MAX_POWER_OF_TWO_IN_INT32 0x40000000
 
-bool
-nsHtml5Tokenizer::EnsureBufferSpace(int32_t aLength)
-{
+bool nsHtml5Tokenizer::EnsureBufferSpace(int32_t aLength) {
   MOZ_RELEASE_ASSERT(aLength >= 0, "Negative length.");
   if (aLength > MAX_POWER_OF_TWO_IN_INT32) {
     // Can't happen when loading from network.
@@ -46,14 +44,14 @@ nsHtml5Tokenizer::EnsureBufferSpace(int32_t aLength)
       worstCase += 1;
     }
     strBuf = jArray<char16_t, int32_t>::newFallibleJArray(
-      mozilla::RoundUpPow2(worstCase.value()));
+        mozilla::RoundUpPow2(worstCase.value()));
     if (!strBuf) {
       return false;
     }
   } else if (worstCase.value() > strBuf.length) {
     jArray<char16_t, int32_t> newBuf =
-      jArray<char16_t, int32_t>::newFallibleJArray(
-        mozilla::RoundUpPow2(worstCase.value()));
+        jArray<char16_t, int32_t>::newFallibleJArray(
+            mozilla::RoundUpPow2(worstCase.value()));
     if (!newBuf) {
       return false;
     }
@@ -63,46 +61,27 @@ nsHtml5Tokenizer::EnsureBufferSpace(int32_t aLength)
   return true;
 }
 
-void
-nsHtml5Tokenizer::StartPlainText()
-{
+void nsHtml5Tokenizer::StartPlainText() {
   stateSave = nsHtml5Tokenizer::PLAINTEXT;
 }
 
-void
-nsHtml5Tokenizer::EnableViewSource(nsHtml5Highlighter* aHighlighter)
-{
-  mViewSource = aHighlighter;
+void nsHtml5Tokenizer::EnableViewSource(nsHtml5Highlighter* aHighlighter) {
+  mViewSource = WrapUnique(aHighlighter);
 }
 
-bool
-nsHtml5Tokenizer::FlushViewSource()
-{
-  return mViewSource->FlushOps();
-}
+bool nsHtml5Tokenizer::FlushViewSource() { return mViewSource->FlushOps(); }
 
-void
-nsHtml5Tokenizer::StartViewSource(const nsAutoString& aTitle)
-{
+void nsHtml5Tokenizer::StartViewSource(const nsAutoString& aTitle) {
   mViewSource->Start(aTitle);
 }
 
-void
-nsHtml5Tokenizer::EndViewSource()
-{
-  mViewSource->End();
-}
+void nsHtml5Tokenizer::EndViewSource() { mViewSource->End(); }
 
-void
-nsHtml5Tokenizer::errWarnLtSlashInRcdata()
-{
-}
+void nsHtml5Tokenizer::errWarnLtSlashInRcdata() {}
 
 // The null checks below annotated MOZ_LIKELY are not actually necessary.
 
-void
-nsHtml5Tokenizer::errUnquotedAttributeValOrNull(char16_t c)
-{
+void nsHtml5Tokenizer::errUnquotedAttributeValOrNull(char16_t c) {
   if (MOZ_LIKELY(mViewSource)) {
     switch (c) {
       case '<':
@@ -122,9 +101,8 @@ nsHtml5Tokenizer::errUnquotedAttributeValOrNull(char16_t c)
   }
 }
 
-void
-nsHtml5Tokenizer::errLtOrEqualsOrGraveInUnquotedAttributeOrNull(char16_t c)
-{
+void nsHtml5Tokenizer::errLtOrEqualsOrGraveInUnquotedAttributeOrNull(
+    char16_t c) {
   if (MOZ_LIKELY(mViewSource)) {
     switch (c) {
       case '=':
@@ -140,9 +118,7 @@ nsHtml5Tokenizer::errLtOrEqualsOrGraveInUnquotedAttributeOrNull(char16_t c)
   }
 }
 
-void
-nsHtml5Tokenizer::errBadCharBeforeAttributeNameOrNull(char16_t c)
-{
+void nsHtml5Tokenizer::errBadCharBeforeAttributeNameOrNull(char16_t c) {
   if (MOZ_LIKELY(mViewSource)) {
     if (c == '<') {
       mViewSource->AddErrorToCurrentNode("errBadCharBeforeAttributeNameLt");
@@ -154,17 +130,13 @@ nsHtml5Tokenizer::errBadCharBeforeAttributeNameOrNull(char16_t c)
   }
 }
 
-void
-nsHtml5Tokenizer::errBadCharAfterLt(char16_t c)
-{
+void nsHtml5Tokenizer::errBadCharAfterLt(char16_t c) {
   if (MOZ_LIKELY(mViewSource)) {
     mViewSource->AddErrorToCurrentNode("errBadCharAfterLt");
   }
 }
 
-void
-nsHtml5Tokenizer::errQuoteOrLtInAttributeNameOrNull(char16_t c)
-{
+void nsHtml5Tokenizer::errQuoteOrLtInAttributeNameOrNull(char16_t c) {
   if (MOZ_LIKELY(mViewSource)) {
     if (c == '<') {
       mViewSource->AddErrorToCurrentNode("errLtInAttributeName");
@@ -174,9 +146,8 @@ nsHtml5Tokenizer::errQuoteOrLtInAttributeNameOrNull(char16_t c)
   }
 }
 
-void
-nsHtml5Tokenizer::maybeErrAttributesOnEndTag(nsHtml5HtmlAttributes* attrs)
-{
+void nsHtml5Tokenizer::maybeErrAttributesOnEndTag(
+    nsHtml5HtmlAttributes* attrs) {
   if (mViewSource && attrs->getLength() != 0) {
     /*
      * When an end tag token is emitted with attributes, that is a parse
@@ -186,407 +157,307 @@ nsHtml5Tokenizer::maybeErrAttributesOnEndTag(nsHtml5HtmlAttributes* attrs)
   }
 }
 
-void
-nsHtml5Tokenizer::maybeErrSlashInEndTag(bool selfClosing)
-{
+void nsHtml5Tokenizer::maybeErrSlashInEndTag(bool selfClosing) {
   if (mViewSource && selfClosing && endTag) {
     mViewSource->AddErrorToCurrentSlash("maybeErrSlashInEndTag");
   }
 }
 
-char16_t
-nsHtml5Tokenizer::errNcrNonCharacter(char16_t ch)
-{
+char16_t nsHtml5Tokenizer::errNcrNonCharacter(char16_t ch) {
   if (MOZ_UNLIKELY(mViewSource)) {
     mViewSource->AddErrorToCurrentNode("errNcrNonCharacter");
   }
   return ch;
 }
 
-void
-nsHtml5Tokenizer::errAstralNonCharacter(int32_t ch)
-{
+void nsHtml5Tokenizer::errAstralNonCharacter(int32_t ch) {
   if (MOZ_UNLIKELY(mViewSource)) {
     mViewSource->AddErrorToCurrentNode("errNcrNonCharacter");
   }
 }
 
-char16_t
-nsHtml5Tokenizer::errNcrControlChar(char16_t ch)
-{
+char16_t nsHtml5Tokenizer::errNcrControlChar(char16_t ch) {
   if (MOZ_UNLIKELY(mViewSource)) {
     mViewSource->AddErrorToCurrentNode("errNcrControlChar");
   }
   return ch;
 }
 
-void
-nsHtml5Tokenizer::errGarbageAfterLtSlash()
-{
+void nsHtml5Tokenizer::errGarbageAfterLtSlash() {
   if (MOZ_LIKELY(mViewSource)) {
     mViewSource->AddErrorToCurrentNode("errGarbageAfterLtSlash");
   }
 }
 
-void
-nsHtml5Tokenizer::errLtSlashGt()
-{
+void nsHtml5Tokenizer::errLtSlashGt() {
   if (MOZ_LIKELY(mViewSource)) {
     mViewSource->AddErrorToCurrentNode("errLtSlashGt");
   }
 }
 
-void
-nsHtml5Tokenizer::errCharRefLacksSemicolon()
-{
+void nsHtml5Tokenizer::errCharRefLacksSemicolon() {
   if (MOZ_UNLIKELY(mViewSource)) {
     mViewSource->AddErrorToCurrentNode("errCharRefLacksSemicolon");
   }
 }
 
-void
-nsHtml5Tokenizer::errNoDigitsInNCR()
-{
+void nsHtml5Tokenizer::errNoDigitsInNCR() {
   if (MOZ_UNLIKELY(mViewSource)) {
     mViewSource->AddErrorToCurrentNode("errNoDigitsInNCR");
   }
 }
 
-void
-nsHtml5Tokenizer::errGtInSystemId()
-{
+void nsHtml5Tokenizer::errGtInSystemId() {
   if (MOZ_LIKELY(mViewSource)) {
     mViewSource->AddErrorToCurrentNode("errGtInSystemId");
   }
 }
 
-void
-nsHtml5Tokenizer::errGtInPublicId()
-{
+void nsHtml5Tokenizer::errGtInPublicId() {
   if (MOZ_LIKELY(mViewSource)) {
     mViewSource->AddErrorToCurrentNode("errGtInPublicId");
   }
 }
 
-void
-nsHtml5Tokenizer::errNamelessDoctype()
-{
+void nsHtml5Tokenizer::errNamelessDoctype() {
   if (MOZ_LIKELY(mViewSource)) {
     mViewSource->AddErrorToCurrentNode("errNamelessDoctype");
   }
 }
 
-void
-nsHtml5Tokenizer::errConsecutiveHyphens()
-{
+void nsHtml5Tokenizer::errConsecutiveHyphens() {
   if (MOZ_UNLIKELY(mViewSource)) {
     mViewSource->AddErrorToCurrentNode("errConsecutiveHyphens");
   }
 }
 
-void
-nsHtml5Tokenizer::errPrematureEndOfComment()
-{
+void nsHtml5Tokenizer::errPrematureEndOfComment() {
   if (MOZ_LIKELY(mViewSource)) {
     mViewSource->AddErrorToCurrentNode("errPrematureEndOfComment");
   }
 }
 
-void
-nsHtml5Tokenizer::errBogusComment()
-{
+void nsHtml5Tokenizer::errBogusComment() {
   if (MOZ_UNLIKELY(mViewSource)) {
     mViewSource->AddErrorToCurrentNode("errBogusComment");
   }
 }
 
-void
-nsHtml5Tokenizer::errSlashNotFollowedByGt()
-{
+void nsHtml5Tokenizer::errSlashNotFollowedByGt() {
   if (MOZ_LIKELY(mViewSource)) {
     mViewSource->AddErrorToCurrentSlash("errSlashNotFollowedByGt");
   }
 }
 
-void
-nsHtml5Tokenizer::errNoSpaceBetweenAttributes()
-{
+void nsHtml5Tokenizer::errNoSpaceBetweenAttributes() {
   if (MOZ_LIKELY(mViewSource)) {
     mViewSource->AddErrorToCurrentNode("errNoSpaceBetweenAttributes");
   }
 }
 
-void
-nsHtml5Tokenizer::errAttributeValueMissing()
-{
+void nsHtml5Tokenizer::errAttributeValueMissing() {
   if (MOZ_LIKELY(mViewSource)) {
     mViewSource->AddErrorToCurrentNode("errAttributeValueMissing");
   }
 }
 
-void
-nsHtml5Tokenizer::errEqualsSignBeforeAttributeName()
-{
+void nsHtml5Tokenizer::errEqualsSignBeforeAttributeName() {
   if (MOZ_LIKELY(mViewSource)) {
     mViewSource->AddErrorToCurrentNode("errEqualsSignBeforeAttributeName");
   }
 }
 
-void
-nsHtml5Tokenizer::errLtGt()
-{
+void nsHtml5Tokenizer::errLtGt() {
   if (MOZ_LIKELY(mViewSource)) {
     mViewSource->AddErrorToCurrentNode("errLtGt");
   }
 }
 
-void
-nsHtml5Tokenizer::errProcessingInstruction()
-{
+void nsHtml5Tokenizer::errProcessingInstruction() {
   if (MOZ_LIKELY(mViewSource)) {
     mViewSource->AddErrorToCurrentNode("errProcessingInstruction");
   }
 }
 
-void
-nsHtml5Tokenizer::errUnescapedAmpersandInterpretedAsCharacterReference()
-{
+void nsHtml5Tokenizer::errUnescapedAmpersandInterpretedAsCharacterReference() {
   if (MOZ_UNLIKELY(mViewSource)) {
     mViewSource->AddErrorToCurrentAmpersand(
-      "errUnescapedAmpersandInterpretedAsCharacterReference");
+        "errUnescapedAmpersandInterpretedAsCharacterReference");
   }
 }
 
-void
-nsHtml5Tokenizer::errNotSemicolonTerminated()
-{
+void nsHtml5Tokenizer::errNotSemicolonTerminated() {
   if (MOZ_UNLIKELY(mViewSource)) {
     mViewSource->AddErrorToCurrentNode("errNotSemicolonTerminated");
   }
 }
 
-void
-nsHtml5Tokenizer::errNoNamedCharacterMatch()
-{
+void nsHtml5Tokenizer::errNoNamedCharacterMatch() {
   if (MOZ_UNLIKELY(mViewSource)) {
     mViewSource->AddErrorToCurrentAmpersand("errNoNamedCharacterMatch");
   }
 }
 
-void
-nsHtml5Tokenizer::errQuoteBeforeAttributeName(char16_t c)
-{
+void nsHtml5Tokenizer::errQuoteBeforeAttributeName(char16_t c) {
   if (MOZ_LIKELY(mViewSource)) {
     mViewSource->AddErrorToCurrentNode("errQuoteBeforeAttributeName");
   }
 }
 
-void
-nsHtml5Tokenizer::errExpectedPublicId()
-{
+void nsHtml5Tokenizer::errExpectedPublicId() {
   if (MOZ_LIKELY(mViewSource)) {
     mViewSource->AddErrorToCurrentNode("errExpectedPublicId");
   }
 }
 
-void
-nsHtml5Tokenizer::errBogusDoctype()
-{
+void nsHtml5Tokenizer::errBogusDoctype() {
   if (MOZ_UNLIKELY(mViewSource)) {
     mViewSource->AddErrorToCurrentNode("errBogusDoctype");
   }
 }
 
-void
-nsHtml5Tokenizer::errNcrSurrogate()
-{
+void nsHtml5Tokenizer::errNcrSurrogate() {
   if (MOZ_UNLIKELY(mViewSource)) {
     mViewSource->AddErrorToCurrentNode("errNcrSurrogate");
   }
 }
 
-void
-nsHtml5Tokenizer::errNcrCr()
-{
+void nsHtml5Tokenizer::errNcrCr() {
   if (MOZ_UNLIKELY(mViewSource)) {
     mViewSource->AddErrorToCurrentNode("errNcrCr");
   }
 }
 
-void
-nsHtml5Tokenizer::errNcrInC1Range()
-{
+void nsHtml5Tokenizer::errNcrInC1Range() {
   if (MOZ_UNLIKELY(mViewSource)) {
     mViewSource->AddErrorToCurrentNode("errNcrInC1Range");
   }
 }
 
-void
-nsHtml5Tokenizer::errEofInPublicId()
-{
+void nsHtml5Tokenizer::errEofInPublicId() {
   if (MOZ_UNLIKELY(mViewSource)) {
     mViewSource->AddErrorToCurrentRun("errEofInPublicId");
   }
 }
 
-void
-nsHtml5Tokenizer::errEofInComment()
-{
+void nsHtml5Tokenizer::errEofInComment() {
   if (MOZ_UNLIKELY(mViewSource)) {
     mViewSource->AddErrorToCurrentRun("errEofInComment");
   }
 }
 
-void
-nsHtml5Tokenizer::errEofInDoctype()
-{
+void nsHtml5Tokenizer::errEofInDoctype() {
   if (MOZ_UNLIKELY(mViewSource)) {
     mViewSource->AddErrorToCurrentRun("errEofInDoctype");
   }
 }
 
-void
-nsHtml5Tokenizer::errEofInAttributeValue()
-{
+void nsHtml5Tokenizer::errEofInAttributeValue() {
   if (MOZ_UNLIKELY(mViewSource)) {
     mViewSource->AddErrorToCurrentRun("errEofInAttributeValue");
   }
 }
 
-void
-nsHtml5Tokenizer::errEofInAttributeName()
-{
+void nsHtml5Tokenizer::errEofInAttributeName() {
   if (MOZ_UNLIKELY(mViewSource)) {
     mViewSource->AddErrorToCurrentRun("errEofInAttributeName");
   }
 }
 
-void
-nsHtml5Tokenizer::errEofWithoutGt()
-{
+void nsHtml5Tokenizer::errEofWithoutGt() {
   if (MOZ_UNLIKELY(mViewSource)) {
     mViewSource->AddErrorToCurrentRun("errEofWithoutGt");
   }
 }
 
-void
-nsHtml5Tokenizer::errEofInTagName()
-{
+void nsHtml5Tokenizer::errEofInTagName() {
   if (MOZ_UNLIKELY(mViewSource)) {
     mViewSource->AddErrorToCurrentRun("errEofInTagName");
   }
 }
 
-void
-nsHtml5Tokenizer::errEofInEndTag()
-{
+void nsHtml5Tokenizer::errEofInEndTag() {
   if (MOZ_UNLIKELY(mViewSource)) {
     mViewSource->AddErrorToCurrentRun("errEofInEndTag");
   }
 }
 
-void
-nsHtml5Tokenizer::errEofAfterLt()
-{
+void nsHtml5Tokenizer::errEofAfterLt() {
   if (MOZ_UNLIKELY(mViewSource)) {
     mViewSource->AddErrorToCurrentRun("errEofAfterLt");
   }
 }
 
-void
-nsHtml5Tokenizer::errNcrOutOfRange()
-{
+void nsHtml5Tokenizer::errNcrOutOfRange() {
   if (MOZ_UNLIKELY(mViewSource)) {
     mViewSource->AddErrorToCurrentNode("errNcrOutOfRange");
   }
 }
 
-void
-nsHtml5Tokenizer::errNcrUnassigned()
-{
+void nsHtml5Tokenizer::errNcrUnassigned() {
   if (MOZ_UNLIKELY(mViewSource)) {
     mViewSource->AddErrorToCurrentNode("errNcrUnassigned");
   }
 }
 
-void
-nsHtml5Tokenizer::errDuplicateAttribute()
-{
+void nsHtml5Tokenizer::errDuplicateAttribute() {
   if (MOZ_UNLIKELY(mViewSource)) {
     mViewSource->AddErrorToCurrentNode("errDuplicateAttribute");
   }
 }
 
-void
-nsHtml5Tokenizer::errEofInSystemId()
-{
+void nsHtml5Tokenizer::errEofInSystemId() {
   if (MOZ_UNLIKELY(mViewSource)) {
     mViewSource->AddErrorToCurrentRun("errEofInSystemId");
   }
 }
 
-void
-nsHtml5Tokenizer::errExpectedSystemId()
-{
+void nsHtml5Tokenizer::errExpectedSystemId() {
   if (MOZ_LIKELY(mViewSource)) {
     mViewSource->AddErrorToCurrentNode("errExpectedSystemId");
   }
 }
 
-void
-nsHtml5Tokenizer::errMissingSpaceBeforeDoctypeName()
-{
+void nsHtml5Tokenizer::errMissingSpaceBeforeDoctypeName() {
   if (MOZ_LIKELY(mViewSource)) {
     mViewSource->AddErrorToCurrentNode("errMissingSpaceBeforeDoctypeName");
   }
 }
 
-void
-nsHtml5Tokenizer::errHyphenHyphenBang()
-{
+void nsHtml5Tokenizer::errNestedComment() {
   if (MOZ_LIKELY(mViewSource)) {
-    mViewSource->AddErrorToCurrentNode("errHyphenHyphenBang");
+    mViewSource->AddErrorToCurrentNode("errNestedComment");
   }
 }
 
-void
-nsHtml5Tokenizer::errNcrControlChar()
-{
+void nsHtml5Tokenizer::errNcrControlChar() {
   if (MOZ_UNLIKELY(mViewSource)) {
     mViewSource->AddErrorToCurrentNode("errNcrControlChar");
   }
 }
 
-void
-nsHtml5Tokenizer::errNcrZero()
-{
+void nsHtml5Tokenizer::errNcrZero() {
   if (MOZ_UNLIKELY(mViewSource)) {
     mViewSource->AddErrorToCurrentNode("errNcrZero");
   }
 }
 
-void
-nsHtml5Tokenizer::errNoSpaceBetweenDoctypeSystemKeywordAndQuote()
-{
+void nsHtml5Tokenizer::errNoSpaceBetweenDoctypeSystemKeywordAndQuote() {
   if (MOZ_LIKELY(mViewSource)) {
     mViewSource->AddErrorToCurrentNode(
-      "errNoSpaceBetweenDoctypeSystemKeywordAndQuote");
+        "errNoSpaceBetweenDoctypeSystemKeywordAndQuote");
   }
 }
 
-void
-nsHtml5Tokenizer::errNoSpaceBetweenPublicAndSystemIds()
-{
+void nsHtml5Tokenizer::errNoSpaceBetweenPublicAndSystemIds() {
   if (MOZ_LIKELY(mViewSource)) {
     mViewSource->AddErrorToCurrentNode("errNoSpaceBetweenPublicAndSystemIds");
   }
 }
 
-void
-nsHtml5Tokenizer::errNoSpaceBetweenDoctypePublicKeywordAndQuote()
-{
+void nsHtml5Tokenizer::errNoSpaceBetweenDoctypePublicKeywordAndQuote() {
   if (MOZ_LIKELY(mViewSource)) {
     mViewSource->AddErrorToCurrentNode(
-      "errNoSpaceBetweenDoctypePublicKeywordAndQuote");
+        "errNoSpaceBetweenDoctypePublicKeywordAndQuote");
   }
 }
