@@ -31,7 +31,7 @@ nsHtml5TreeBuilder::nsHtml5TreeBuilder(nsHtml5OplessBuilder* aBuilder)
       headPointer(nullptr),
       charBufferLen(0),
       quirks(false),
-      isSrcdocDocument(false),
+      forceNoQuirks(false),
       mBuilder(aBuilder),
       mViewSource(nullptr),
       mOpSink(nullptr),
@@ -70,7 +70,7 @@ nsHtml5TreeBuilder::nsHtml5TreeBuilder(nsAHtml5TreeOpSink* aOpSink,
       headPointer(nullptr),
       charBufferLen(0),
       quirks(false),
-      isSrcdocDocument(false),
+      forceNoQuirks(false),
       mBuilder(nullptr),
       mViewSource(nullptr),
       mOpSink(aOpSink),
@@ -147,13 +147,13 @@ nsIContentHandle* nsHtml5TreeBuilder::createElement(
     opCreateHTMLElement opeation(
         content, aName, aAttributes, aCreator.html, aIntendedParent,
         (!!mSpeculativeLoadStage) ? mozilla::dom::FROM_PARSER_NETWORK
-                                  : dom::FROM_PARSER_DOCUMENT_WRITE);
+                                  : mozilla::dom::FROM_PARSER_DOCUMENT_WRITE);
     treeOp->Init(mozilla::AsVariant(opeation));
   } else if (aNamespace == kNameSpaceID_SVG) {
     opCreateSVGElement operation(
         content, aName, aAttributes, aCreator.svg, aIntendedParent,
         (!!mSpeculativeLoadStage) ? mozilla::dom::FROM_PARSER_NETWORK
-                                  : dom::FROM_PARSER_DOCUMENT_WRITE);
+                                  : mozilla::dom::FROM_PARSER_DOCUMENT_WRITE);
     treeOp->Init(mozilla::AsVariant(operation));
   } else {
     // kNameSpaceID_MathML
@@ -186,7 +186,8 @@ nsIContentHandle* nsHtml5TreeBuilder::createElement(
             nsHtml5String sizes =
                 aAttributes->getValue(nsHtml5AttributeName::ATTR_SIZES);
             mSpeculativeLoadQueue.AppendElement()->InitImage(
-                url, crossOrigin, referrerPolicy, srcset, sizes, false);
+                url, crossOrigin, /* aMedia = */ nullptr, referrerPolicy,
+                srcset, sizes, false);
           }
         } else if (nsGkAtoms::source == aName) {
           nsHtml5String srcset =
@@ -236,9 +237,9 @@ nsIContentHandle* nsHtml5TreeBuilder::createElement(
             bool noModule =
                 aAttributes->contains(nsHtml5AttributeName::ATTR_NOMODULE);
             mSpeculativeLoadQueue.AppendElement()->InitScript(
-                url, charset, type, crossOrigin, integrity, referrerPolicy,
-                mode == nsHtml5TreeBuilder::IN_HEAD, async, defer, noModule,
-                false);
+                url, charset, type, crossOrigin, /* aMedia = */ nullptr,
+                integrity, referrerPolicy, mode == nsHtml5TreeBuilder::IN_HEAD,
+                async, defer, noModule, false);
             mCurrentHtmlScriptIsAsyncOrDefer = async || defer;
           }
         } else if (nsGkAtoms::link == aName) {
@@ -259,8 +260,10 @@ nsIContentHandle* nsHtml5TreeBuilder::createElement(
                     aAttributes->getValue(nsHtml5AttributeName::ATTR_INTEGRITY);
                 nsHtml5String referrerPolicy = aAttributes->getValue(
                     nsHtml5AttributeName::ATTR_REFERRERPOLICY);
+                nsHtml5String media =
+                    aAttributes->getValue(nsHtml5AttributeName::ATTR_MEDIA);
                 mSpeculativeLoadQueue.AppendElement()->InitStyle(
-                    url, charset, crossOrigin, referrerPolicy, integrity,
+                    url, charset, crossOrigin, media, referrerPolicy, integrity,
                     false);
               }
             } else if (rel.LowerCaseEqualsASCII("preconnect")) {
@@ -287,6 +290,8 @@ nsIContentHandle* nsHtml5TreeBuilder::createElement(
                     aAttributes->getValue(nsHtml5AttributeName::ATTR_INTEGRITY);
                 nsHtml5String referrerPolicy = aAttributes->getValue(
                     nsHtml5AttributeName::ATTR_REFERRERPOLICY);
+                nsHtml5String media =
+                    aAttributes->getValue(nsHtml5AttributeName::ATTR_MEDIA);
 
                 // Note that respective speculative loaders for scripts and
                 // styles check all additional attributes to be equal to use the
@@ -299,26 +304,27 @@ nsIContentHandle* nsHtml5TreeBuilder::createElement(
                   nsHtml5String type =
                       aAttributes->getValue(nsHtml5AttributeName::ATTR_TYPE);
                   mSpeculativeLoadQueue.AppendElement()->InitScript(
-                      url, charset, type, crossOrigin, integrity,
+                      url, charset, type, crossOrigin, media, integrity,
                       referrerPolicy, mode == nsHtml5TreeBuilder::IN_HEAD,
                       false, false, false, true);
                 } else if (as.LowerCaseEqualsASCII("style")) {
                   mSpeculativeLoadQueue.AppendElement()->InitStyle(
-                      url, charset, crossOrigin, referrerPolicy, integrity,
-                      true);
+                      url, charset, crossOrigin, media, referrerPolicy,
+                      integrity, true);
                 } else if (as.LowerCaseEqualsASCII("image")) {
                   nsHtml5String srcset = aAttributes->getValue(
                       nsHtml5AttributeName::ATTR_IMAGESRCSET);
                   nsHtml5String sizes = aAttributes->getValue(
                       nsHtml5AttributeName::ATTR_IMAGESIZES);
                   mSpeculativeLoadQueue.AppendElement()->InitImage(
-                      url, crossOrigin, referrerPolicy, srcset, sizes, true);
+                      url, crossOrigin, media, referrerPolicy, srcset, sizes,
+                      true);
                 } else if (as.LowerCaseEqualsASCII("font")) {
                   mSpeculativeLoadQueue.AppendElement()->InitFont(
-                      url, crossOrigin, referrerPolicy);
+                      url, crossOrigin, media, referrerPolicy);
                 } else if (as.LowerCaseEqualsASCII("fetch")) {
                   mSpeculativeLoadQueue.AppendElement()->InitFetch(
-                      url, crossOrigin, referrerPolicy);
+                      url, crossOrigin, media, referrerPolicy);
                 }
                 // Other "as" values will be supported later.
               }
@@ -329,7 +335,7 @@ nsIContentHandle* nsHtml5TreeBuilder::createElement(
               aAttributes->getValue(nsHtml5AttributeName::ATTR_POSTER);
           if (url) {
             mSpeculativeLoadQueue.AppendElement()->InitImage(
-                url, nullptr, nullptr, nullptr, nullptr, false);
+                url, nullptr, nullptr, nullptr, nullptr, nullptr, false);
           }
         } else if (nsGkAtoms::style == aName) {
           mImportScanner.Start();
@@ -385,7 +391,7 @@ nsIContentHandle* nsHtml5TreeBuilder::createElement(
           }
           if (url) {
             mSpeculativeLoadQueue.AppendElement()->InitImage(
-                url, nullptr, nullptr, nullptr, nullptr, false);
+                url, nullptr, nullptr, nullptr, nullptr, nullptr, false);
           }
         } else if (nsGkAtoms::script == aName) {
           nsHtml5TreeOperation* treeOp =
@@ -414,9 +420,9 @@ nsIContentHandle* nsHtml5TreeBuilder::createElement(
             nsHtml5String referrerPolicy = aAttributes->getValue(
                 nsHtml5AttributeName::ATTR_REFERRERPOLICY);
             mSpeculativeLoadQueue.AppendElement()->InitScript(
-                url, nullptr, type, crossOrigin, integrity, referrerPolicy,
-                mode == nsHtml5TreeBuilder::IN_HEAD, false, false, false,
-                false);
+                url, nullptr, type, crossOrigin, /* aMedia = */ nullptr,
+                integrity, referrerPolicy, mode == nsHtml5TreeBuilder::IN_HEAD,
+                false, false, false, false);
           }
         } else if (nsGkAtoms::style == aName) {
           mImportScanner.Start();
@@ -608,7 +614,7 @@ void nsHtml5TreeBuilder::appendElement(nsIContentHandle* aChild,
   if (mBuilder) {
     nsresult rv = nsHtml5TreeOperation::Append(
         static_cast<nsIContent*>(aChild), static_cast<nsIContent*>(aParent),
-        mBuilder);
+        mozilla::dom::FROM_PARSER_FRAGMENT, mBuilder);
     if (NS_FAILED(rv)) {
       MarkAsBrokenAndRequestSuspensionWithBuilder(rv);
     }
@@ -620,7 +626,11 @@ void nsHtml5TreeBuilder::appendElement(nsIContentHandle* aChild,
     MarkAsBrokenAndRequestSuspensionWithoutBuilder(NS_ERROR_OUT_OF_MEMORY);
     return;
   }
-  opAppend operation(aChild, aParent);
+
+  opAppend operation(aChild, aParent,
+                     (!!mSpeculativeLoadStage)
+                         ? mozilla::dom::FROM_PARSER_NETWORK
+                         : mozilla::dom::FROM_PARSER_DOCUMENT_WRITE);
   treeOp->Init(mozilla::AsVariant(operation));
 }
 
@@ -1355,6 +1365,7 @@ void nsHtml5TreeBuilder::StartPlainTextViewSource(const nsAutoString& aTitle) {
 
 void nsHtml5TreeBuilder::StartPlainText() {
   MOZ_ASSERT(!mBuilder, "Must not view source with builder.");
+  setForceNoQuirks(true);
   startTag(nsHtml5ElementName::ELT_LINK,
            nsHtml5PlainTextUtils::NewLinkAttributes(), false);
 
@@ -1494,13 +1505,13 @@ void nsHtml5TreeBuilder::errStrayDoctype() {
 }
 
 void nsHtml5TreeBuilder::errAlmostStandardsDoctype() {
-  if (MOZ_UNLIKELY(mViewSource) && !isSrcdocDocument) {
+  if (MOZ_UNLIKELY(mViewSource) && !forceNoQuirks) {
     mViewSource->AddErrorToCurrentRun("errAlmostStandardsDoctype");
   }
 }
 
 void nsHtml5TreeBuilder::errQuirkyDoctype() {
-  if (MOZ_UNLIKELY(mViewSource) && !isSrcdocDocument) {
+  if (MOZ_UNLIKELY(mViewSource) && !forceNoQuirks) {
     mViewSource->AddErrorToCurrentRun("errQuirkyDoctype");
   }
 }
@@ -1548,7 +1559,7 @@ void nsHtml5TreeBuilder::errFooBetweenHeadAndBody(nsAtom* aName) {
 }
 
 void nsHtml5TreeBuilder::errStartTagWithoutDoctype() {
-  if (MOZ_UNLIKELY(mViewSource) && !isSrcdocDocument) {
+  if (MOZ_UNLIKELY(mViewSource) && !forceNoQuirks) {
     mViewSource->AddErrorToCurrentRun("errStartTagWithoutDoctype");
   }
 }
@@ -1638,7 +1649,7 @@ void nsHtml5TreeBuilder::errStartTagInTableBody(nsAtom* aName) {
 }
 
 void nsHtml5TreeBuilder::errEndTagSeenWithoutDoctype() {
-  if (MOZ_UNLIKELY(mViewSource) && !isSrcdocDocument) {
+  if (MOZ_UNLIKELY(mViewSource) && !forceNoQuirks) {
     mViewSource->AddErrorToCurrentRun("errEndTagSeenWithoutDoctype");
   }
 }
@@ -1735,3 +1746,9 @@ void nsHtml5TreeBuilder::errEndWithUnclosedElements(nsAtom* aName) {
     mViewSource->AddErrorToCurrentRun("errEndWithUnclosedElements", aName);
   }
 }
+
+void nsHtml5TreeBuilder::errListUnclosedStartTags(int32_t aIgnored) {
+  if (MOZ_UNLIKELY(mViewSource)) {
+    mViewSource->AddErrorToCurrentRun("errListUnclosedStartTags");
+  }
+}
