@@ -15,14 +15,20 @@ echo "running as" $(id)
 
 : MOZHARNESS_SCRIPT             ${MOZHARNESS_SCRIPT}
 : MOZHARNESS_CONFIG             ${MOZHARNESS_CONFIG}
+: MOZHARNESS_CONFIG_PATHS       ${MOZHARNESS_CONFIG_PATHS}
 : MOZHARNESS_ACTIONS            ${MOZHARNESS_ACTIONS}
 : MOZHARNESS_OPTIONS            ${MOZHARNESS_OPTIONS}
 
 : TOOLTOOL_CACHE                ${TOOLTOOL_CACHE:=/builds/worker/tooltool-cache}
 
+: MOZ_SCM_LEVEL                 ${MOZ_SCM_LEVEL:=1}
+
 : NEED_XVFB                     ${NEED_XVFB:=false}
 
+: MOZ_SCM_LEVEL                 ${MOZ_SCM_LEVEL:=1}
+
 : WORKSPACE                     ${WORKSPACE:=/builds/worker/workspace}
+: MOZ_OBJDIR                    ${MOZ_OBJDIR:=$WORKSPACE/obj-build}
 
 set -v
 
@@ -33,15 +39,11 @@ fail() {
 }
 
 export MOZ_CRASHREPORTER_NO_REPORT=1
-export MOZ_OBJDIR=obj-firefox
 export TINDERBOX_OUTPUT=1
 
-# Ensure that in tree libraries can be found
-export LIBRARY_PATH=$LIBRARY_PATH:$WORKSPACE/src/obj-firefox:$WORKSPACE/src/gcc/lib64
-
 # test required parameters are supplied
 if [[ -z ${MOZHARNESS_SCRIPT} ]]; then fail "MOZHARNESS_SCRIPT is not set"; fi
-if [[ -z ${MOZHARNESS_CONFIG} ]]; then fail "MOZHARNESS_CONFIG is not set"; fi
+if [[ -z "${MOZHARNESS_CONFIG}" && -z "${EXTRA_MOZHARNESS_CONFIG}" ]]; then fail "MOZHARNESS_CONFIG or EXTRA_MOZHARNESS_CONFIG is not set"; fi
 
 cleanup() {
     local rv=$?
@@ -62,6 +64,13 @@ fi
 # entirely effective.
 export TOOLTOOL_CACHE
 
+export MOZ_OBJDIR
+
+config_path_cmds=""
+for path in ${MOZHARNESS_CONFIG_PATHS}; do
+    config_path_cmds="${config_path_cmds} --extra-config-path ${GECKO_PATH}/${path}"
+done
+
 # support multiple, space delimited, config files
 config_cmds=""
 for cfg in $MOZHARNESS_CONFIG; do
@@ -78,7 +87,6 @@ if [ -n "$MOZHARNESS_ACTIONS" ]; then
 fi
 
 # if MOZHARNESS_OPTIONS is given, append them to mozharness command line run
-# e.g. enable-pgo
 if [ -n "$MOZHARNESS_OPTIONS" ]; then
     options=""
     for option in $MOZHARNESS_OPTIONS; do
@@ -88,11 +96,13 @@ fi
 
 cd /builds/worker
 
-python2.7 $WORKSPACE/build/src/testing/${MOZHARNESS_SCRIPT} \
-  --disable-mock \
-  --revision ${GECKO_HEAD_REV} \
+$GECKO_PATH/mach python \
+  --requirements $GECKO_PATH/build/psutil_requirements.txt \
+  -- \
+  $GECKO_PATH/testing/${MOZHARNESS_SCRIPT} \
+  ${config_path_cmds} \
+  ${config_cmds} \
   $actions \
   $options \
-  ${config_cmds} \
   --log-level=debug \
-  --work-dir=$WORKSPACE/build \
+  --work-dir=$WORKSPACE \
