@@ -14,7 +14,7 @@
 module.exports = {
   env: {
     browser: true,
-    es6: true,
+    es2021: true,
     "mozilla/privileged": true,
   },
 
@@ -77,7 +77,7 @@ module.exports = {
   ],
 
   parserOptions: {
-    ecmaVersion: 11,
+    ecmaVersion: 12,
   },
 
   // When adding items to this file please check for effects on sub-directories.
@@ -129,8 +129,9 @@ module.exports = {
     "mozilla/no-useless-removeEventListener": "error",
     "mozilla/prefer-boolean-length-check": "error",
     "mozilla/prefer-formatValues": "error",
-    "mozilla/reject-chromeutils-import-null": "error",
+    "mozilla/reject-chromeutils-import-params": "error",
     "mozilla/reject-importGlobalProperties": ["error", "allownonwebidl"],
+    "mozilla/reject-osfile": "warn",
     "mozilla/rejects-requires-await": "error",
     "mozilla/use-cc-etc": "error",
     "mozilla/use-chromeutils-generateqi": "error",
