@@ -10,11 +10,13 @@
 #include "platform.h"
 #include "ProfileBuffer.h"
 #include "ProfilerBacktrace.h"
-#include "ProfilerMarkerPayload.h"
+#include "ProfilerRustBindings.h"
 
+#include "js/ProfilingFrameIterator.h"
 #include "jsapi.h"
 #include "jsfriendapi.h"
 #include "mozilla/Logging.h"
+#include "mozilla/ScopeExit.h"
 #include "mozilla/Sprintf.h"
 #include "mozilla/StackWalk.h"
 #include "nsThreadUtils.h"
@@ -68,6 +70,13 @@ ProfileBufferEntry::ProfileBufferEntry(Kind aKind, uint64_t aUint64)
   memcpy(mStorage, &aUint64, sizeof(aUint64));
 }
 
+ProfileBufferEntry::ProfileBufferEntry(Kind aKind, ProfilerThreadId aThreadId)
+    : mKind(aKind) {
+  static_assert(std::is_trivially_copyable_v<ProfilerThreadId>);
+  static_assert(sizeof(aThreadId) <= sizeof(mStorage));
+  memcpy(mStorage, &aThreadId, sizeof(aThreadId));
+}
+
 const char* ProfileBufferEntry::GetString() const {
   const char* result;
   memcpy(&result, mStorage, sizeof(result));
@@ -104,6 +113,13 @@ uint64_t ProfileBufferEntry::GetUint64() const {
   return result;
 }
 
+ProfilerThreadId ProfileBufferEntry::GetThreadId() const {
+  ProfilerThreadId result;
+  static_assert(std::is_trivially_copyable_v<ProfilerThreadId>);
+  memcpy(&result, mStorage, sizeof(result));
+  return result;
+}
+
 void ProfileBufferEntry::CopyCharsInto(char (&aOutArray)[kNumChars]) const {
   memcpy(aOutArray, mStorage, kNumChars);
 }
@@ -169,6 +185,11 @@ class MOZ_RAII AutoArraySchemaWriter {
     mJSONWriter.DoubleElement(aValue);
   }
 
+  void TimeMsElement(uint32_t aIndex, double aTime_ms) {
+    FillUpTo(aIndex);
+    mJSONWriter.TimeDoubleMsElement(aTime_ms);
+  }
+
   void BoolElement(uint32_t aIndex, bool aValue) {
     FillUpTo(aIndex);
     mJSONWriter.BoolElement(aValue);
@@ -517,28 +538,50 @@ void JITFrameInfo::AddInfoForRange(
 }
 
 struct ProfileSample {
-  uint32_t mStack;
-  double mTime;
+  uint32_t mStack = 0;
+  bool mStackIsEmpty = false;
+  double mTime = 0.0;
   Maybe<double> mResponsiveness;
+  RunningTimes mRunningTimes;
 };
 
+// Write CPU measurements with "Delta" unit, which is some amount of work that
+// happened since the previous sample.
+static void WriteDelta(AutoArraySchemaWriter& aSchemaWriter, uint32_t aProperty,
+                       uint64_t aDelta) {
+  aSchemaWriter.IntElement(aProperty, int64_t(aDelta));
+}
+
 static void WriteSample(SpliceableJSONWriter& aWriter,
                         const ProfileSample& aSample) {
   enum Schema : uint32_t {
     STACK = 0,
     TIME = 1,
-    EVENT_DELAY = 2,
+    EVENT_DELAY = 2
+#define RUNNING_TIME_SCHEMA(index, name, unit, jsonProperty) , name
+    PROFILER_FOR_EACH_RUNNING_TIME(RUNNING_TIME_SCHEMA)
+#undef RUNNING_TIME_SCHEMA
   };
 
   AutoArraySchemaWriter writer(aWriter);
 
   writer.IntElement(STACK, aSample.mStack);
 
-  writer.DoubleElement(TIME, aSample.mTime);
+  writer.TimeMsElement(TIME, aSample.mTime);
 
   if (aSample.mResponsiveness.isSome()) {
     writer.DoubleElement(EVENT_DELAY, *aSample.mResponsiveness);
   }
+
+#define RUNNING_TIME_STREAM(index, name, unit, jsonProperty) \
+  aSample.mRunningTimes.Get##name##unit().apply(             \
+      [&writer](const uint64_t& aValue) {                    \
+        Write##unit(writer, name, aValue);                   \
+      });
+
+  PROFILER_FOR_EACH_RUNNING_TIME(RUNNING_TIME_STREAM)
+
+#undef RUNNING_TIME_STREAM
 }
 
 class EntryGetter {
@@ -626,6 +669,7 @@ class EntryGetter {
 //   ( /* Samples */
 //     ThreadId
 //     TimeBeforeCompactStack
+//     RunningTimes?
 //     UnresponsivenessDurationMs?
 //     CompactStack
 //         /* internally including:
@@ -636,7 +680,12 @@ class EntryGetter {
 //           )+
 //         */
 //   )
-//   | MarkerData
+//   | ( /* Reference to a previous identical sample */
+//       ThreadId
+//       TimeBeforeSameSample
+//       RunningTimes?
+//       SameSample
+//     )
 //   | Marker
 //   | ( /* Counters */
 //       CounterId
@@ -754,9 +803,9 @@ class EntryGetter {
     continue;                                              \
   }
 
-int ProfileBuffer::StreamSamplesToJSON(SpliceableJSONWriter& aWriter,
-                                       int aThreadId, double aSinceTime,
-                                       UniqueStacks& aUniqueStacks) const {
+ProfilerThreadId ProfileBuffer::StreamSamplesToJSON(
+    SpliceableJSONWriter& aWriter, ProfilerThreadId aThreadId,
+    double aSinceTime, UniqueStacks& aUniqueStacks) const {
   UniquePtr<char[]> dynStrBuf = MakeUnique<char[]>(kMaxFrameKeyLength);
 
   return mEntries.Read([&](ProfileChunkedBuffer::Reader* aReader) {
@@ -764,7 +813,12 @@ int ProfileBuffer::StreamSamplesToJSON(SpliceableJSONWriter& aWriter,
                "ProfileChunkedBuffer cannot be out-of-session when sampler is "
                "running");
 
-    int processedThreadId = 0;
+    ProfilerThreadId processedThreadId;
+
+    // This ProfileSample object gets filled with relevant data related to each
+    // sample. Parts of it may be reused from one sample to the next, in
+    // particular when stacks are identical in `SameSample` entries.
+    ProfileSample sample;
 
     EntryGetter e(*aReader);
 
@@ -795,21 +849,21 @@ int ProfileBuffer::StreamSamplesToJSON(SpliceableJSONWriter& aWriter,
       // must be a ThreadId entry.
       MOZ_ASSERT(e.Get().IsThreadId());
 
-      int threadId = e.Get().GetInt();
+      ProfilerThreadId threadId = e.Get().GetThreadId();
       e.Next();
 
       // Ignore samples that are for the wrong thread.
-      if (threadId != aThreadId && aThreadId != 0) {
+      if (threadId != aThreadId && aThreadId.IsSpecified()) {
         continue;
       }
 
-      MOZ_ASSERT(aThreadId != 0 || processedThreadId == 0,
-                 "aThreadId==0 should only be used with 1-sample buffer");
-
-      ProfileSample sample;
+      MOZ_ASSERT(
+          aThreadId.IsSpecified() || !processedThreadId.IsSpecified(),
+          "Unspecified aThreadId should only be used with 1-sample buffer");
 
       auto ReadStack = [&](EntryGetter& e, uint64_t entryPosition,
-                           const Maybe<double>& unresponsiveDuration) {
+                           const Maybe<double>& unresponsiveDuration,
+                           const RunningTimes& aRunningTimes) {
         UniqueStacks::StackKey stack =
             aUniqueStacks.BeginStack(UniqueStacks::FrameKey("(root)"));
 
@@ -958,20 +1012,25 @@ int ProfileBuffer::StreamSamplesToJSON(SpliceableJSONWriter& aWriter,
           }
         }
 
-        if (numFrames == 0) {
+        // Even if this stack is considered empty, it contains the root frame,
+        // which needs to be in the JSON output because following "same samples"
+        // may refer to it when reusing this sample.mStack.
+        sample.mStack = aUniqueStacks.GetOrAddStackIndex(stack);
+        sample.mStackIsEmpty = (numFrames == 0);
+
+        if (sample.mStackIsEmpty && aRunningTimes.IsEmpty()) {
           // It is possible to have empty stacks if native stackwalking is
-          // disabled. Skip samples with empty stacks. (See Bug 1497985).
-          // Thus, don't use ERROR_AND_CONTINUE, but just continue by returning
-          // from this lambda.
+          // disabled. Skip samples with empty stacks, unless we have useful
+          // running times.
           return;
         }
 
-        sample.mStack = aUniqueStacks.GetOrAddStackIndex(stack);
-
         if (unresponsiveDuration.isSome()) {
           sample.mResponsiveness = unresponsiveDuration;
         }
 
+        sample.mRunningTimes = aRunningTimes;
+
         WriteSample(aWriter, sample);
 
         processedThreadId = threadId;
@@ -986,7 +1045,7 @@ int ProfileBuffer::StreamSamplesToJSON(SpliceableJSONWriter& aWriter,
           continue;
         }
 
-        ReadStack(e, 0, Nothing{});
+        ReadStack(e, 0, Nothing{}, RunningTimes{});
       } else if (e.Has() && e.Get().IsTimeBeforeCompactStack()) {
         sample.mTime = e.Get().GetDouble();
 
@@ -996,6 +1055,7 @@ int ProfileBuffer::StreamSamplesToJSON(SpliceableJSONWriter& aWriter,
           continue;
         }
 
+        RunningTimes runningTimes;
         Maybe<double> unresponsiveDuration;
 
         ProfileChunkedBuffer::BlockIterator it = e.Iterator();
@@ -1008,6 +1068,12 @@ int ProfileBuffer::StreamSamplesToJSON(SpliceableJSONWriter& aWriter,
           ProfileBufferEntry::Kind kind =
               er.ReadObject<ProfileBufferEntry::Kind>();
 
+          // There may be running times before the CompactStack.
+          if (kind == ProfileBufferEntry::Kind::RunningTimes) {
+            er.ReadIntoObject(runningTimes);
+            continue;
+          }
+
           // There may be an UnresponsiveDurationMs before the CompactStack.
           if (kind == ProfileBufferEntry::Kind::UnresponsiveDurationMs) {
             unresponsiveDuration = Some(er.ReadObject<double>());
@@ -1023,11 +1089,9 @@ int ProfileBuffer::StreamSamplesToJSON(SpliceableJSONWriter& aWriter,
               MOZ_ASSERT(aReader,
                          "Local ProfileChunkedBuffer cannot be out-of-session");
               EntryGetter stackEntryGetter(*aReader);
-              if (stackEntryGetter.Has()) {
-                ReadStack(stackEntryGetter,
-                          it.CurrentBlockIndex().ConvertToProfileBufferIndex(),
-                          unresponsiveDuration);
-              }
+              ReadStack(stackEntryGetter,
+                        it.CurrentBlockIndex().ConvertToProfileBufferIndex(),
+                        unresponsiveDuration, runningTimes);
             });
             mWorkerChunkManager.Reset(tempBuffer.GetAllChunks());
             break;
@@ -1039,6 +1103,63 @@ int ProfileBuffer::StreamSamplesToJSON(SpliceableJSONWriter& aWriter,
           er.SetRemainingBytes(0);
         }
 
+        e.Next();
+      } else if (e.Has() && e.Get().IsTimeBeforeSameSample()) {
+        if (sample.mTime == 0.0) {
+          // We don't have any full sample yet, we cannot duplicate a "previous"
+          // one. This should only happen at most once per thread, for the very
+          // first sample.
+          continue;
+        }
+
+        // Keep the same `mStack` as previously output.
+        // Note that it may be empty, this is checked below before writing it.
+        (void)sample.mStack;
+
+        sample.mTime = e.Get().GetDouble();
+
+        // Ignore samples that are too old.
+        if (sample.mTime < aSinceTime) {
+          e.Next();
+          continue;
+        }
+
+        sample.mResponsiveness = Nothing{};
+
+        sample.mRunningTimes.Clear();
+
+        ProfileChunkedBuffer::BlockIterator it = e.Iterator();
+        for (;;) {
+          ++it;
+          if (it.IsAtEnd()) {
+            break;
+          }
+          ProfileBufferEntryReader er = *it;
+          ProfileBufferEntry::Kind kind =
+              er.ReadObject<ProfileBufferEntry::Kind>();
+
+          // There may be running times before the SameSample.
+          if (kind == ProfileBufferEntry::Kind::RunningTimes) {
+            er.ReadIntoObject(sample.mRunningTimes);
+            continue;
+          }
+
+          if (kind == ProfileBufferEntry::Kind::SameSample) {
+            if (sample.mStackIsEmpty && sample.mRunningTimes.IsEmpty()) {
+              // Skip samples with empty stacks, unless we have useful running
+              // times.
+              break;
+            }
+            WriteSample(aWriter, sample);
+            break;
+          }
+
+          MOZ_ASSERT(kind >= ProfileBufferEntry::Kind::LEGACY_LIMIT,
+                     "There should be no legacy entries between "
+                     "TimeBeforeSameSample and SameSample");
+          er.SetRemainingBytes(0);
+        }
+
         e.Next();
       } else {
         ERROR_AND_CONTINUE("expected a Time entry");
@@ -1049,7 +1170,8 @@ int ProfileBuffer::StreamSamplesToJSON(SpliceableJSONWriter& aWriter,
   });
 }
 
-void ProfileBuffer::AddJITInfoForRange(uint64_t aRangeStart, int aThreadId,
+void ProfileBuffer::AddJITInfoForRange(uint64_t aRangeStart,
+                                       ProfilerThreadId aThreadId,
                                        JSContext* aContext,
                                        JITFrameInfo& aJITFrameInfo) const {
   // We can only process JitReturnAddr entries if we have a JSContext.
@@ -1079,7 +1201,7 @@ void ProfileBuffer::AddJITInfoForRange(uint64_t aRangeStart, int aThreadId,
             }
 
             MOZ_ASSERT(e.Get().IsThreadId());
-            int threadId = e.Get().GetInt();
+            ProfilerThreadId threadId = e.Get().GetThreadId();
             e.Next();
 
             // Ignore samples that are for a different thread.
@@ -1135,6 +1257,9 @@ void ProfileBuffer::AddJITInfoForRange(uint64_t aRangeStart, int aThreadId,
               }
 
               e.Next();
+            } else if (e.Has() && e.Get().IsTimeBeforeSameSample()) {
+              // Sample index, nothing to do.
+
             } else {
               ERROR_AND_CONTINUE("expected a Time entry");
             }
@@ -1144,7 +1269,7 @@ void ProfileBuffer::AddJITInfoForRange(uint64_t aRangeStart, int aThreadId,
 }
 
 void ProfileBuffer::StreamMarkersToJSON(SpliceableJSONWriter& aWriter,
-                                        int aThreadId,
+                                        ProfilerThreadId aThreadId,
                                         const TimeStamp& aProcessStartTime,
                                         double aSinceTime,
                                         UniqueStacks& aUniqueStacks) const {
@@ -1154,78 +1279,44 @@ void ProfileBuffer::StreamMarkersToJSON(SpliceableJSONWriter& aWriter,
     MOZ_ASSERT(static_cast<ProfileBufferEntry::KindUnderlyingType>(type) <
                static_cast<ProfileBufferEntry::KindUnderlyingType>(
                    ProfileBufferEntry::Kind::MODERN_LIMIT));
-    // Code should *return* from the switch if the entry was fully read.
-    // Code should *break* from the switch if the entry was not fully read (we
-    // then need to adjust the reader position to the end of the entry, as
-    // expected by the reader code.)
-    switch (type) {
-      case ProfileBufferEntry::Kind::MarkerData:
-        if (aER.ReadObject<int>() != aThreadId) {
-          break;  // Entry not fully read.
-        }
-        aWriter.StartArrayElement();
-        {
-          // Extract the information from the buffer:
-          // Each entry is made up of the following:
-          //
-          // [
-          //   ProfileBufferEntry::Kind::MarkerData, <- already read
-          //   threadId,                             <- already read
-          //   name,                                 <- next location in entries
-          //   startTime,
-          //   endTime,
-          //   phase,
-          //   categoryPair,
-          //   payload
-          // ]
-          auto name = aER.ReadObject<std::string>();
-          auto startTime = aER.ReadObject<double>();
-          auto endTime = aER.ReadObject<double>();
-          auto phase = aER.ReadObject<uint8_t>();
-          const JS::ProfilingCategoryPairInfo& info =
-              GetProfilingCategoryPairInfo(
-                  static_cast<JS::ProfilingCategoryPair>(
-                      aER.ReadObject<uint32_t>()));
-          auto payload = aER.ReadObject<UniquePtr<ProfilerMarkerPayload>>();
+    bool entryWasFullyRead = false;
 
-          MOZ_ASSERT(aER.RemainingBytes() == 0);
+    if (type == ProfileBufferEntry::Kind::Marker) {
+      entryWasFullyRead =
+          mozilla::base_profiler_markers_detail::DeserializeAfterKindAndStream(
+              aER, aWriter, aThreadId,
+              [&](ProfileChunkedBuffer& aChunkedBuffer) {
+                ProfilerBacktrace backtrace("", &aChunkedBuffer);
+                backtrace.StreamJSON(aWriter, aProcessStartTime, aUniqueStacks);
+              },
+              [&](mozilla::base_profiler_markers_detail::Streaming::
+                      DeserializerTag aTag) {
+                size_t payloadSize = aER.RemainingBytes();
 
-          // Now write this information to JSON with the following schema:
-          // [name, startTime, endTime, phase, category, data]
-          aUniqueStacks.mUniqueStrings->WriteElement(aWriter, name);
-          aWriter.DoubleElement(startTime);
-          aWriter.DoubleElement(endTime);
-          aWriter.IntElement(phase);
-          aWriter.IntElement(unsigned(info.mCategory));
-          if (payload) {
-            aWriter.StartObjectElement(SpliceableJSONWriter::SingleLineStyle);
-            {
-              payload->StreamPayload(aWriter, aProcessStartTime, aUniqueStacks);
-            }
-            aWriter.EndObject();
-          }
-        }
-        aWriter.EndArray();
-        return;  // Entry fully read.
-
-      case ProfileBufferEntry::Kind::Marker:
-        if (mozilla::base_profiler_markers_detail::
-                DeserializeAfterKindAndStream(
-                    aER, aWriter, aThreadId,
-                    [&](ProfileChunkedBuffer& aChunkedBuffer) {
-                      ProfilerBacktrace backtrace("", &aChunkedBuffer);
-                      backtrace.StreamJSON(aWriter, aProcessStartTime,
-                                           aUniqueStacks);
-                    })) {
-          return;  // Entry fully read.
-        }
-        break;  // Entry not fully read.
-
-      default:
-        break;  // Entry not fully read.
+                ProfileBufferEntryReader::DoubleSpanOfConstBytes spans =
+                    aER.ReadSpans(payloadSize);
+                if (MOZ_LIKELY(spans.IsSingleSpan())) {
+                  // Only a single span, we can just refer to it directly
+                  // instead of copying it.
+                  profiler::ffi::gecko_profiler_serialize_marker_for_tag(
+                      aTag, spans.mFirstOrOnly.Elements(), payloadSize,
+                      &aWriter);
+                } else {
+                  // Two spans, we need to concatenate them by copying.
+                  uint8_t* payloadBuffer = new uint8_t[payloadSize];
+                  spans.CopyBytesTo(payloadBuffer);
+                  profiler::ffi::gecko_profiler_serialize_marker_for_tag(
+                      aTag, payloadBuffer, payloadSize, &aWriter);
+                  delete[] payloadBuffer;
+                }
+              });
     }
 
-    aER.SetRemainingBytes(0);
+    if (!entryWasFullyRead) {
+      // The entry was not a marker, or it was a marker for another thread.
+      // We probably didn't read the whole entry, so we need to skip to the end.
+      aER.SetRemainingBytes(0);
+    }
   });
 }
 
@@ -1315,7 +1406,7 @@ void ProfileBuffer::StreamProfilerOverheadToJSON(
           threads.Count(thread);
 
           AutoArraySchemaWriter writer(aWriter);
-          writer.DoubleElement(TIME, time);
+          writer.TimeMsElement(TIME, time);
           writer.DoubleElement(LOCKING, locking);
           writer.DoubleElement(MARKER_CLEANING, cleaning);
           writer.DoubleElement(COUNTERS, counter);
@@ -1527,7 +1618,7 @@ void ProfileBuffer::StreamCountersToJSON(SpliceableJSONWriter& aWriter,
                          uint64_t(std::numeric_limits<int64_t>::max()));
 
               AutoArraySchemaWriter writer(aWriter);
-              writer.DoubleElement(TIME, samples[i].mTime);
+              writer.TimeMsElement(TIME, samples[i].mTime);
               writer.IntElement(
                   NUMBER,
                   static_cast<int64_t>(samples[i].mNumber - previousNumber));
@@ -1555,12 +1646,12 @@ static void AddPausedRange(SpliceableJSONWriter& aWriter, const char* aReason,
                            const Maybe<double>& aEndTime) {
   aWriter.Start();
   if (aStartTime) {
-    aWriter.DoubleProperty("startTime", *aStartTime);
+    aWriter.TimeDoubleMsProperty("startTime", *aStartTime);
   } else {
     aWriter.NullProperty("startTime");
   }
   if (aEndTime) {
-    aWriter.DoubleProperty("endTime", *aEndTime);
+    aWriter.TimeDoubleMsProperty("endTime", *aEndTime);
   } else {
     aWriter.NullProperty("endTime");
   }
@@ -1608,13 +1699,62 @@ void ProfileBuffer::StreamPausedRangesToJSON(SpliceableJSONWriter& aWriter,
   });
 }
 
-bool ProfileBuffer::DuplicateLastSample(int aThreadId,
-                                        const TimeStamp& aProcessStartTime,
-                                        Maybe<uint64_t>& aLastSample) {
+bool ProfileBuffer::DuplicateLastSample(ProfilerThreadId aThreadId,
+                                        double aSampleTimeMs,
+                                        Maybe<uint64_t>& aLastSample,
+                                        const RunningTimes& aRunningTimes) {
   if (!aLastSample) {
     return false;
   }
 
+  if (mEntries.IsIndexInCurrentChunk(ProfileBufferIndex{*aLastSample})) {
+    // The last (fully-written) sample is in this chunk, we can refer to it.
+
+    // Note that between now and when we write the SameSample below, another
+    // chunk could have been started, so the SameSample will in fact refer to a
+    // block in a previous chunk. This is okay, because:
+    // - When serializing to JSON, if that chunk is still there, we'll still be
+    //   able to find that old stack, so nothing will be lost.
+    // - If unfortunately that chunk has been destroyed, we will lose this
+    //   sample. But this will only happen to the first sample (per thread) in
+    //   in the whole JSON output, because the next time we're here to duplicate
+    //   the same sample again, IsIndexInCurrentChunk will say `false` and we
+    //   will fall back to the normal copy or even re-sample. Losing the first
+    //   sample out of many in a whole recording is acceptable.
+    //
+    // |---| = chunk, S = Sample, D = Duplicate, s = same sample
+    // |---S-s-s--| |s-D--s--s-| |s-D--s---s|
+    // Later, the first chunk is destroyed/recycled:
+    //              |s-D--s--s-| |s-D--s---s| |-...
+    // Output:       ^ ^  ^       ^
+    //               `-|--|-------|--- Same but no previous -> lost.
+    //                 `--|-------|--- Full duplicate sample.
+    //                    `-------|--- Same with previous -> okay.
+    //                            `--- Same but now we have a previous -> okay!
+
+    AUTO_PROFILER_STATS(DuplicateLastSample_SameSample);
+
+    // Add the thread id first. We don't update `aLastSample` because we are not
+    // writing a full sample.
+    (void)AddThreadIdEntry(aThreadId);
+
+    // Copy the new time, to be followed by a SameSample.
+    AddEntry(ProfileBufferEntry::TimeBeforeSameSample(aSampleTimeMs));
+
+    // Add running times if they have data.
+    if (!aRunningTimes.IsEmpty()) {
+      mEntries.PutObjects(ProfileBufferEntry::Kind::RunningTimes,
+                          aRunningTimes);
+    }
+
+    // Finish with a SameSample entry.
+    mEntries.PutObjects(ProfileBufferEntry::Kind::SameSample);
+
+    return true;
+  }
+
+  AUTO_PROFILER_STATS(DuplicateLastSample_copy);
+
   ProfileChunkedBuffer tempBuffer(
       ProfileChunkedBuffer::ThreadSafety::WithoutMutex, mWorkerChunkManager);
 
@@ -1636,7 +1776,7 @@ bool ProfileBuffer::DuplicateLastSample(int aThreadId,
     }
 
     MOZ_RELEASE_ASSERT(e.Has() && e.Get().IsThreadId() &&
-                       e.Get().GetInt() == aThreadId);
+                       e.Get().GetThreadId() == aThreadId);
 
     e.Next();
 
@@ -1651,21 +1791,23 @@ bool ProfileBuffer::DuplicateLastSample(int aThreadId,
         case ProfileBufferEntry::Kind::CollectionStart:
         case ProfileBufferEntry::Kind::CollectionEnd:
         case ProfileBufferEntry::Kind::ThreadId:
+        case ProfileBufferEntry::Kind::TimeBeforeSameSample:
           // We're done.
           return true;
         case ProfileBufferEntry::Kind::Time:
           // Copy with new time
-          AddEntry(tempBuffer,
-                   ProfileBufferEntry::Time(
-                       (TimeStamp::NowUnfuzzed() - aProcessStartTime)
-                           .ToMilliseconds()));
+          AddEntry(tempBuffer, ProfileBufferEntry::Time(aSampleTimeMs));
           break;
         case ProfileBufferEntry::Kind::TimeBeforeCompactStack: {
           // Copy with new time, followed by a compact stack.
           AddEntry(tempBuffer,
-                   ProfileBufferEntry::TimeBeforeCompactStack(
-                       (TimeStamp::NowUnfuzzed() - aProcessStartTime)
-                           .ToMilliseconds()));
+                   ProfileBufferEntry::TimeBeforeCompactStack(aSampleTimeMs));
+
+          // Add running times if they have data.
+          if (!aRunningTimes.IsEmpty()) {
+            tempBuffer.PutObjects(ProfileBufferEntry::Kind::RunningTimes,
+                                  aRunningTimes);
+          }
 
           // The `CompactStack` *must* be present afterwards, but may not
           // immediately follow `TimeBeforeCompactStack` (e.g., some markers
@@ -1688,11 +1830,13 @@ bool ProfileBuffer::DuplicateLastSample(int aThreadId,
               // Found our CompactStack, just make a copy of the whole entry.
               er = *it;
               auto bytes = er.RemainingBytes();
-              MOZ_ASSERT(bytes < 65536);
+              MOZ_ASSERT(bytes <
+                         ProfileBufferChunkManager::scExpectedMaximumStackSize);
               tempBuffer.Put(bytes, [&](Maybe<ProfileBufferEntryWriter>& aEW) {
                 MOZ_ASSERT(aEW.isSome(), "tempBuffer cannot be out-of-session");
                 aEW->WriteFromReader(er, bytes);
               });
+              // CompactStack marks the end, we're done.
               break;
             }
 
