@@ -7,99 +7,59 @@
 #ifndef ProfileBufferEntry_h
 #define ProfileBufferEntry_h
 
-#include <ostream>
-#include "GeckoProfiler.h"
-#include "platform.h"
-#include "ProfileJSONWriter.h"
-#include "ProfilerBacktrace.h"
-#include "mozilla/RefPtr.h"
-#include <string>
-#include <map>
-#include "js/ProfilingFrameIterator.h"
-#include "js/TrackedOptimizationInfo.h"
-#include "nsHashKeys.h"
-#include "nsDataHashtable.h"
-#include "mozilla/Maybe.h"
-#include "mozilla/Vector.h"
+#include <cstdint>
+#include <cstdlib>
+#include <functional>
+#include <utility>
 #include "gtest/MozGtestFriend.h"
+#include "js/ProfilingCategory.h"
+#include "mozilla/Attributes.h"
 #include "mozilla/HashFunctions.h"
+#include "mozilla/HashTable.h"
+#include "mozilla/Maybe.h"
+#include "mozilla/ProfileBufferEntryKinds.h"
+#include "mozilla/ProfileJSONWriter.h"
+#include "mozilla/ProfilerUtils.h"
 #include "mozilla/UniquePtr.h"
-#include "nsClassHashtable.h"
 #include "mozilla/Variant.h"
-#include "nsTArray.h"
+#include "mozilla/Vector.h"
+#include "nsString.h"
 
-class ProfilerMarker;
-
-// NOTE!  If you add entries, you need to verify if they need to be added to the
-// switch statement in DuplicateLastSample!
-#define FOR_EACH_PROFILE_BUFFER_ENTRY_KIND(MACRO)                   \
-  MACRO(Category, int)                                              \
-  MACRO(CollectionStart, double)                                    \
-  MACRO(CollectionEnd, double)                                      \
-  MACRO(Label, const char*)                                         \
-  MACRO(FrameFlags, uint64_t)                                       \
-  MACRO(DynamicStringFragment, char*) /* char[kNumChars], really */ \
-  MACRO(JitReturnAddr, void*)                                       \
-  MACRO(LineNumber, int)                                            \
-  MACRO(ColumnNumber, int)                                          \
-  MACRO(NativeLeafAddr, void*)                                      \
-  MACRO(Marker, ProfilerMarker*)                                    \
-  MACRO(Pause, double)                                              \
-  MACRO(Responsiveness, double)                                     \
-  MACRO(Resume, double)                                             \
-  MACRO(ThreadId, int)                                              \
-  MACRO(Time, double)                                               \
-  MACRO(ResidentMemory, uint64_t)                                   \
-  MACRO(UnsharedMemory, uint64_t)                                   \
-  MACRO(CounterId, void*)                                           \
-  MACRO(CounterKey, uint64_t)                                       \
-  MACRO(Number, uint64_t)                                           \
-  MACRO(Count, int64_t)
-
-// NB: Packing this structure has been shown to cause SIGBUS issues on ARM.
-#if !defined(GP_ARCH_arm)
-#pragma pack(push, 1)
-#endif
+class ProfilerCodeAddressService;
+struct JSContext;
 
 class ProfileBufferEntry {
  public:
-  enum class Kind : uint8_t {
-    INVALID = 0,
-#define KIND(k, t) k,
-    FOR_EACH_PROFILE_BUFFER_ENTRY_KIND(KIND)
-#undef KIND
-        LIMIT
-  };
+  using KindUnderlyingType = mozilla::ProfileBufferEntryKindUnderlyingType;
+  using Kind = mozilla::ProfileBufferEntryKind;
 
   ProfileBufferEntry();
 
-  // This is equal to sizeof(double), which is the largest non-char variant in
-  // |u|.
-  static const size_t kNumChars = 8;
+  static constexpr size_t kNumChars = mozilla::ProfileBufferEntryNumChars;
 
  private:
   // aString must be a static string.
   ProfileBufferEntry(Kind aKind, const char* aString);
   ProfileBufferEntry(Kind aKind, char aChars[kNumChars]);
   ProfileBufferEntry(Kind aKind, void* aPtr);
-  ProfileBufferEntry(Kind aKind, ProfilerMarker* aMarker);
   ProfileBufferEntry(Kind aKind, double aDouble);
   ProfileBufferEntry(Kind aKind, int64_t aInt64);
   ProfileBufferEntry(Kind aKind, uint64_t aUint64);
   ProfileBufferEntry(Kind aKind, int aInt);
+  ProfileBufferEntry(Kind aKind, ProfilerThreadId aThreadId);
 
  public:
-#define CTOR(k, t)                            \
-  static ProfileBufferEntry k(t aVal) {       \
-    return ProfileBufferEntry(Kind::k, aVal); \
+#define CTOR(KIND, TYPE, SIZE)                   \
+  static ProfileBufferEntry KIND(TYPE aVal) {    \
+    return ProfileBufferEntry(Kind::KIND, aVal); \
   }
   FOR_EACH_PROFILE_BUFFER_ENTRY_KIND(CTOR)
 #undef CTOR
 
   Kind GetKind() const { return mKind; }
 
-#define IS_KIND(k, t) \
-  bool Is##k() const { return mKind == Kind::k; }
+#define IS_KIND(KIND, TYPE, SIZE) \
+  bool Is##KIND() const { return mKind == Kind::KIND; }
   FOR_EACH_PROFILE_BUFFER_ENTRY_KIND(IS_KIND)
 #undef IS_KIND
 
@@ -112,48 +72,20 @@ class ProfileBufferEntry {
   friend class ProfileBuffer;
 
   Kind mKind;
-  union {
-    const char* mString;
-    char mChars[kNumChars];
-    void* mPtr;
-    ProfilerMarker* mMarker;
-    double mDouble;
-    int mInt;
-    int64_t mInt64;
-    uint64_t mUint64;
-  } u;
+  uint8_t mStorage[kNumChars];
+
+  const char* GetString() const;
+  void* GetPtr() const;
+  double GetDouble() const;
+  int GetInt() const;
+  int64_t GetInt64() const;
+  uint64_t GetUint64() const;
+  ProfilerThreadId GetThreadId() const;
+  void CopyCharsInto(char (&aOutArray)[kNumChars]) const;
 };
 
-#if !defined(GP_ARCH_arm)
 // Packed layout: 1 byte for the tag + 8 bytes for the value.
 static_assert(sizeof(ProfileBufferEntry) == 9, "bad ProfileBufferEntry size");
-#pragma pack(pop)
-#endif
-
-class UniqueJSONStrings {
- public:
-  UniqueJSONStrings();
-  explicit UniqueJSONStrings(const UniqueJSONStrings& aOther);
-
-  void SpliceStringTableElements(SpliceableJSONWriter& aWriter) {
-    aWriter.TakeAndSplice(mStringTableWriter.WriteFunc());
-  }
-
-  void WriteProperty(mozilla::JSONWriter& aWriter, const char* aName,
-                     const char* aStr) {
-    aWriter.IntProperty(aName, GetOrAddIndex(aStr));
-  }
-
-  void WriteElement(mozilla::JSONWriter& aWriter, const char* aStr) {
-    aWriter.IntElement(GetOrAddIndex(aStr));
-  }
-
-  uint32_t GetOrAddIndex(const char* aStr);
-
- private:
-  SpliceableChunkedJSONWriter mStringTableWriter;
-  nsDataHashtable<nsCStringHashKey, uint32_t> mStringToIndexMap;
-};
 
 // Contains all the information about JIT frames that is needed to stream stack
 // frames for JitReturnAddr entries in the profiler buffer.
@@ -169,8 +101,10 @@ struct JITFrameInfoForBufferRange final {
   uint64_t mRangeEnd;  // mRangeEnd marks the first invalid index.
 
   struct JITFrameKey {
-    uint32_t Hash() const;
-    bool operator==(const JITFrameKey& aOther) const;
+    bool operator==(const JITFrameKey& aOther) const {
+      return mCanonicalAddress == aOther.mCanonicalAddress &&
+             mDepth == aOther.mDepth;
+    }
     bool operator!=(const JITFrameKey& aOther) const {
       return !(*this == aOther);
     }
@@ -178,10 +112,31 @@ struct JITFrameInfoForBufferRange final {
     void* mCanonicalAddress;
     uint32_t mDepth;
   };
-  nsClassHashtable<nsPtrHashKey<void>, nsTArray<JITFrameKey>>
-      mJITAddressToJITFramesMap;
-  nsClassHashtable<nsGenericHashKey<JITFrameKey>, nsCString>
-      mJITFrameToFrameJSONMap;
+  struct JITFrameKeyHasher {
+    using Lookup = JITFrameKey;
+
+    static mozilla::HashNumber hash(const JITFrameKey& aLookup) {
+      mozilla::HashNumber hash = 0;
+      hash = mozilla::AddToHash(hash, aLookup.mCanonicalAddress);
+      hash = mozilla::AddToHash(hash, aLookup.mDepth);
+      return hash;
+    }
+
+    static bool match(const JITFrameKey& aKey, const JITFrameKey& aLookup) {
+      return aKey == aLookup;
+    }
+
+    static void rekey(JITFrameKey& aKey, const JITFrameKey& aNewKey) {
+      aKey = aNewKey;
+    }
+  };
+
+  using JITAddressToJITFramesMap =
+      mozilla::HashMap<void*, mozilla::Vector<JITFrameKey>>;
+  JITAddressToJITFramesMap mJITAddressToJITFramesMap;
+  using JITFrameToFrameJSONMap =
+      mozilla::HashMap<JITFrameKey, nsCString, JITFrameKeyHasher>;
+  JITFrameToFrameJSONMap mJITFrameToFrameJSONMap;
 };
 
 // Contains JITFrameInfoForBufferRange objects for multiple profiler buffer
@@ -207,19 +162,19 @@ struct JITFrameInfo final {
   // Returns whether the information stored in this object is still relevant
   // for any entries in the buffer.
   bool HasExpired(uint64_t aCurrentBufferRangeStart) const {
-    if (mRanges.IsEmpty()) {
+    if (mRanges.empty()) {
       // No information means no relevant information. Allow this object to be
       // discarded.
       return true;
     }
-    return mRanges.LastElement().mRangeEnd <= aCurrentBufferRangeStart;
+    return mRanges.back().mRangeEnd <= aCurrentBufferRangeStart;
   }
 
   // The array of ranges of JIT frame information, sorted by buffer position.
   // Ranges are non-overlapping.
   // The JSON of the cached frames can contain string indexes, which refer
   // to strings in mUniqueStrings.
-  nsTArray<JITFrameInfoForBufferRange> mRanges;
+  mozilla::Vector<JITFrameInfoForBufferRange> mRanges;
 
   // The string table which contains strings used in the frame JSON that's
   // cached in mRanges.
@@ -230,15 +185,16 @@ class UniqueStacks {
  public:
   struct FrameKey {
     explicit FrameKey(const char* aLocation)
-        : mData(NormalFrameData{nsCString(aLocation), false, mozilla::Nothing(),
-                                mozilla::Nothing()}) {}
+        : mData(NormalFrameData{nsCString(aLocation), false, false, 0,
+                                mozilla::Nothing(), mozilla::Nothing()}) {}
 
-    FrameKey(nsCString&& aLocation, bool aRelevantForJS,
-             const mozilla::Maybe<unsigned>& aLine,
+    FrameKey(nsCString&& aLocation, bool aRelevantForJS, bool aBaselineInterp,
+             uint64_t aInnerWindowID, const mozilla::Maybe<unsigned>& aLine,
              const mozilla::Maybe<unsigned>& aColumn,
-             const mozilla::Maybe<unsigned>& aCategory)
-        : mData(NormalFrameData{aLocation, aRelevantForJS, aLine, aColumn,
-                                aCategory}) {}
+             const mozilla::Maybe<JS::ProfilingCategoryPair>& aCategoryPair)
+        : mData(NormalFrameData{aLocation, aRelevantForJS, aBaselineInterp,
+                                aInnerWindowID, aLine, aColumn,
+                                aCategoryPair}) {}
 
     FrameKey(void* aJITAddress, uint32_t aJITDepth, uint32_t aRangeIndex)
         : mData(JITFrameData{aJITAddress, aJITDepth, aRangeIndex}) {}
@@ -255,9 +211,11 @@ class UniqueStacks {
 
       nsCString mLocation;
       bool mRelevantForJS;
+      bool mBaselineInterp;
+      uint64_t mInnerWindowID;
       mozilla::Maybe<unsigned> mLine;
       mozilla::Maybe<unsigned> mColumn;
-      mozilla::Maybe<unsigned> mCategory;
+      mozilla::Maybe<JS::ProfilingCategoryPair> mCategoryPair;
     };
     struct JITFrameData {
       bool operator==(const JITFrameData& aOther) const;
@@ -269,6 +227,50 @@ class UniqueStacks {
     mozilla::Variant<NormalFrameData, JITFrameData> mData;
   };
 
+  struct FrameKeyHasher {
+    using Lookup = FrameKey;
+
+    static mozilla::HashNumber hash(const FrameKey& aLookup) {
+      mozilla::HashNumber hash = 0;
+      if (aLookup.mData.is<FrameKey::NormalFrameData>()) {
+        const FrameKey::NormalFrameData& data =
+            aLookup.mData.as<FrameKey::NormalFrameData>();
+        if (!data.mLocation.IsEmpty()) {
+          hash = mozilla::AddToHash(hash,
+                                    mozilla::HashString(data.mLocation.get()));
+        }
+        hash = mozilla::AddToHash(hash, data.mRelevantForJS);
+        hash = mozilla::AddToHash(hash, data.mBaselineInterp);
+        hash = mozilla::AddToHash(hash, data.mInnerWindowID);
+        if (data.mLine.isSome()) {
+          hash = mozilla::AddToHash(hash, *data.mLine);
+        }
+        if (data.mColumn.isSome()) {
+          hash = mozilla::AddToHash(hash, *data.mColumn);
+        }
+        if (data.mCategoryPair.isSome()) {
+          hash = mozilla::AddToHash(hash,
+                                    static_cast<uint32_t>(*data.mCategoryPair));
+        }
+      } else {
+        const FrameKey::JITFrameData& data =
+            aLookup.mData.as<FrameKey::JITFrameData>();
+        hash = mozilla::AddToHash(hash, data.mCanonicalAddress);
+        hash = mozilla::AddToHash(hash, data.mDepth);
+        hash = mozilla::AddToHash(hash, data.mRangeIndex);
+      }
+      return hash;
+    }
+
+    static bool match(const FrameKey& aKey, const FrameKey& aLookup) {
+      return aKey == aLookup;
+    }
+
+    static void rekey(FrameKey& aKey, const FrameKey& aNewKey) {
+      aKey = aNewKey;
+    }
+  };
+
   struct StackKey {
     mozilla::Maybe<uint32_t> mPrefixStackIndex;
     uint32_t mFrameIndex;
@@ -282,7 +284,7 @@ class UniqueStacks {
           mFrameIndex(aFrame),
           mHash(mozilla::AddToHash(aPrefix.mHash, aFrame)) {}
 
-    uint32_t Hash() const { return mHash; }
+    mozilla::HashNumber Hash() const { return mHash; }
 
     bool operator==(const StackKey& aOther) const {
       return mPrefixStackIndex == aOther.mPrefixStackIndex &&
@@ -290,29 +292,45 @@ class UniqueStacks {
     }
 
    private:
-    uint32_t mHash;
+    mozilla::HashNumber mHash;
+  };
+
+  struct StackKeyHasher {
+    using Lookup = StackKey;
+
+    static mozilla::HashNumber hash(const StackKey& aLookup) {
+      return aLookup.Hash();
+    }
+
+    static bool match(const StackKey& aKey, const StackKey& aLookup) {
+      return aKey == aLookup;
+    }
+
+    static void rekey(StackKey& aKey, const StackKey& aNewKey) {
+      aKey = aNewKey;
+    }
   };
 
   explicit UniqueStacks(JITFrameInfo&& aJITFrameInfo);
 
   // Return a StackKey for aFrame as the stack's root frame (no prefix).
-  MOZ_MUST_USE StackKey BeginStack(const FrameKey& aFrame);
+  [[nodiscard]] StackKey BeginStack(const FrameKey& aFrame);
 
   // Return a new StackKey that is obtained by appending aFrame to aStack.
-  MOZ_MUST_USE StackKey AppendFrame(const StackKey& aStack,
-                                    const FrameKey& aFrame);
+  [[nodiscard]] StackKey AppendFrame(const StackKey& aStack,
+                                     const FrameKey& aFrame);
 
   // Look up frame keys for the given JIT address, and ensure that our frame
   // table has entries for the returned frame keys. The JSON for these frames
   // is taken from mJITInfoRanges.
   // aBufferPosition is needed in order to look up the correct JIT frame info
   // object in mJITInfoRanges.
-  MOZ_MUST_USE mozilla::Maybe<nsTArray<UniqueStacks::FrameKey>>
+  [[nodiscard]] mozilla::Maybe<mozilla::Vector<UniqueStacks::FrameKey>>
   LookupFramesForJITAddressFromBufferPos(void* aJITAddress,
                                          uint64_t aBufferPosition);
 
-  MOZ_MUST_USE uint32_t GetOrAddFrameIndex(const FrameKey& aFrame);
-  MOZ_MUST_USE uint32_t GetOrAddStackIndex(const StackKey& aStack);
+  [[nodiscard]] uint32_t GetOrAddFrameIndex(const FrameKey& aFrame);
+  [[nodiscard]] uint32_t GetOrAddStackIndex(const StackKey& aStack);
 
   void SpliceFrameTableElements(SpliceableJSONWriter& aWriter);
   void SpliceStackTableElements(SpliceableJSONWriter& aWriter);
@@ -324,14 +342,16 @@ class UniqueStacks {
  public:
   mozilla::UniquePtr<UniqueJSONStrings> mUniqueStrings;
 
+  ProfilerCodeAddressService* mCodeAddressService = nullptr;
+
  private:
   SpliceableChunkedJSONWriter mFrameTableWriter;
-  nsDataHashtable<nsGenericHashKey<FrameKey>, uint32_t> mFrameToIndexMap;
+  mozilla::HashMap<FrameKey, uint32_t, FrameKeyHasher> mFrameToIndexMap;
 
   SpliceableChunkedJSONWriter mStackTableWriter;
-  nsDataHashtable<nsGenericHashKey<StackKey>, uint32_t> mStackToIndexMap;
+  mozilla::HashMap<StackKey, uint32_t, StackKeyHasher> mStackToIndexMap;
 
-  nsTArray<JITFrameInfoForBufferRange> mJITInfoRanges;
+  mozilla::Vector<JITFrameInfoForBufferRange> mJITInfoRanges;
 };
 
 //
@@ -360,11 +380,12 @@ class UniqueStacks {
 //     {
 //       "stack": 0,          /* index into stackTable */
 //       "time": 1,           /* number */
-//       "responsiveness": 2, /* number */
+//       "eventDelay": 2,     /* number */
+//       "ThreadCPUDelta": 3, /* optional number */
 //     },
 //     "data":
 //     [
-//       [ 1, 0.0, 0.0 ]      /* { stack: 1, time: 0.0, responsiveness: 0.0 } */
+//       [ 1, 0.0, 0.0 ]      /* { stack: 1, time: 0.0, eventDelay: 0.0 } */
 //     ]
 //   },
 //
@@ -401,16 +422,20 @@ class UniqueStacks {
 //     "schema":
 //     {
 //       "location": 0,       /* index into stringTable */
-//       "implementation": 1, /* index into stringTable */
-//       "optimizations": 2,  /* arbitrary JSON */
-//       "line": 3,           /* number */
-//       "column": 4,         /* number */
-//       "category": 5        /* number */
+//       "relevantForJS": 1,  /* bool */
+//       "innerWindowID": 2,  /* inner window ID of global JS `window` object */
+//       "implementation": 3, /* index into stringTable */
+//       "optimizations": 4,  /* arbitrary JSON */
+//       "line": 5,           /* number */
+//       "column": 6,         /* number */
+//       "category": 7,       /* index into profile.meta.categories */
+//       "subcategory": 8     /* index into
+//       profile.meta.categories[category].subcategories */
 //     },
 //     "data":
 //     [
 //       [ 0 ],               /* { location: '(root)' } */
-//       [ 1, 2 ]             /* { location: 'foo.js',
+//       [ 1, null, null, 2 ] /* { location: 'foo.js',
 //                                 implementation: 'baseline' } */
 //     ]
 //   },
@@ -462,24 +487,6 @@ class UniqueStacks {
 //     },
 //     /* more counters */
 //   ],
-//   "memory":
-//   {
-//     "initial_heap": 12345678,
-//     "samples:
-//     {
-//       "schema":
-//       {
-//         "time": 1,            /* number */
-//         "rss": 2,             /* number */
-//         "uss": 3              /* number */
-//       },
-//       "data":
-//       [
-//         /* { time: 0.1, rss: 12345678, uss: 87654321} */
-//         [ 0.1, 12345678, 87654321 ]
-//       ]
-//     },
-//   },
 // }
 //
 #endif /* ndef ProfileBufferEntry_h */
