@@ -8,15 +8,24 @@
 #define ProfiledThreadData_h
 
 #include "platform.h"
-#include "ProfileBufferEntry.h"
-#include "ThreadInfo.h"
 
-#include "js/ProfilingStack.h"
+#include "mozilla/Maybe.h"
+#include "mozilla/ProfilerThreadRegistrationInfo.h"
 #include "mozilla/TimeStamp.h"
 #include "mozilla/UniquePtr.h"
-#include "nsIEventTarget.h"
+#include "mozilla/RefPtr.h"
+#include "nsStringFwd.h"
 
 class ProfileBuffer;
+class ProfilerCodeAddressService;
+class UniqueStacks;
+class nsIEventTarget;
+struct JITFrameInfo;
+struct JSContext;
+
+namespace mozilla::baseprofiler {
+class SpliceableJSONWriter;
+}
 
 // This class contains information about a thread that is only relevant while
 // the profiler is running, for any threads (both alive and dead) whose thread
@@ -43,7 +52,9 @@ class ProfileBuffer;
 // when the profiler is stopped.
 class ProfiledThreadData final {
  public:
-  ProfiledThreadData(ThreadInfo* aThreadInfo, nsIEventTarget* aEventTarget);
+  ProfiledThreadData(
+      const mozilla::profiler::ThreadRegistrationInfo& aThreadInfo,
+      nsIEventTarget* aEventTarget);
   ~ProfiledThreadData();
 
   void NotifyUnregistered(uint64_t aBufferPosition) {
@@ -51,8 +62,9 @@ class ProfiledThreadData final {
     MOZ_ASSERT(!mBufferPositionWhenReceivedJSContext,
                "JSContext should have been cleared before the thread was "
                "unregistered");
-    mUnregisterTime = mozilla::TimeStamp::NowUnfuzzed();
+    mUnregisterTime = mozilla::TimeStamp::Now();
     mBufferPositionWhenUnregistered = mozilla::Some(aBufferPosition);
+    mPreviousThreadRunningTimes.Clear();
   }
   mozilla::Maybe<uint64_t> BufferPositionWhenUnregistered() {
     return mBufferPositionWhenUnregistered;
@@ -61,16 +73,19 @@ class ProfiledThreadData final {
   mozilla::Maybe<uint64_t>& LastSample() { return mLastSample; }
 
   void StreamJSON(const ProfileBuffer& aBuffer, JSContext* aCx,
-                  SpliceableJSONWriter& aWriter, const nsACString& aProcessName,
-                  const nsACString& aETLDplus1,
+                  mozilla::baseprofiler::SpliceableJSONWriter& aWriter,
+                  const nsACString& aProcessName, const nsACString& aETLDplus1,
                   const mozilla::TimeStamp& aProcessStartTime,
                   double aSinceTime, bool aJSTracerEnabled,
                   ProfilerCodeAddressService* aService);
 
-  void StreamTraceLoggerJSON(JSContext* aCx, SpliceableJSONWriter& aWriter,
-                             const mozilla::TimeStamp& aProcessStartTime);
+  void StreamTraceLoggerJSON(
+      JSContext* aCx, mozilla::baseprofiler::SpliceableJSONWriter& aWriter,
+      const mozilla::TimeStamp& aProcessStartTime);
 
-  const RefPtr<ThreadInfo> Info() const { return mThreadInfo; }
+  const mozilla::profiler::ThreadRegistrationInfo& Info() const {
+    return mThreadInfo;
+  }
 
   void NotifyReceivedJSContext(uint64_t aCurrentBufferPosition) {
     mBufferPositionWhenReceivedJSContext =
@@ -83,13 +98,18 @@ class ProfiledThreadData final {
                                   const mozilla::TimeStamp& aProcessStartTime,
                                   ProfileBuffer& aBuffer);
 
+  RunningTimes& PreviousThreadRunningTimesRef() {
+    return mPreviousThreadRunningTimes;
+  }
+
  private:
   // Group A:
   // The following fields are interesting for the entire lifetime of a
   // ProfiledThreadData object.
 
-  // This thread's thread info.
-  const RefPtr<ThreadInfo> mThreadInfo;
+  // This thread's thread info. Local copy because the one in ThreadRegistration
+  // may be destroyed while ProfiledThreadData stays alive.
+  const mozilla::profiler::ThreadRegistrationInfo mThreadInfo;
 
   // Contains JSON for JIT frames from any JSContexts that were used for this
   // thread in the past.
@@ -99,7 +119,7 @@ class ProfiledThreadData final {
 
   // Group B:
   // The following fields are only used while this thread is alive and
-  // registered. They become Nothing() once the thread is unregistered.
+  // registered. They become Nothing() or empty once the thread is unregistered.
 
   // When sampling, this holds the position in ActivePS::mBuffer of the most
   // recent sample for this thread, or Nothing() if there is no sample for this
@@ -109,6 +129,9 @@ class ProfiledThreadData final {
   // Only non-Nothing() if the thread currently has a JSContext.
   mozilla::Maybe<uint64_t> mBufferPositionWhenReceivedJSContext;
 
+  // RunningTimes at the previous sample if any, or empty.
+  RunningTimes mPreviousThreadRunningTimes;
+
   // Group C:
   // The following fields are only used once this thread has been unregistered.
 
@@ -119,14 +142,13 @@ class ProfiledThreadData final {
 // Stream all samples and markers from aBuffer with the given aThreadId (or 0
 // for everything, which is assumed to be a single backtrace sample.)
 // Returns the thread id of the output sample(s), or 0 if none was present.
-int StreamSamplesAndMarkers(const char* aName, int aThreadId,
-                            const ProfileBuffer& aBuffer,
-                            SpliceableJSONWriter& aWriter,
-                            const nsACString& aProcessName,
-                            const nsACString& aETLDplus1,
-                            const mozilla::TimeStamp& aProcessStartTime,
-                            const mozilla::TimeStamp& aRegisterTime,
-                            const mozilla::TimeStamp& aUnregisterTime,
-                            double aSinceTime, UniqueStacks& aUniqueStacks);
+ProfilerThreadId StreamSamplesAndMarkers(
+    const char* aName, ProfilerThreadId aThreadId, const ProfileBuffer& aBuffer,
+    mozilla::baseprofiler::SpliceableJSONWriter& aWriter,
+    const nsACString& aProcessName, const nsACString& aETLDplus1,
+    const mozilla::TimeStamp& aProcessStartTime,
+    const mozilla::TimeStamp& aRegisterTime,
+    const mozilla::TimeStamp& aUnregisterTime, double aSinceTime,
+    UniqueStacks& aUniqueStacks);
 
 #endif  // ProfiledThreadData_h
